function [A,Bu,Bl,E,b,c,a]=assemblyP1bP1CN(p,t,nu,alpha,f,w)
% Navier-Stokes problem in 2D with P1-bubble/P1 elements for Newton linearization.
% For Laplace operator.
% Loop over triangles.
% 
% Input:
% p ... fem nodes
% t ... fem triangles (counterclockwise oriented)
% alpha ... constant at mass matrix 
% nu ... viscosity constant
% f ... force at nodes
% w ... given velocity vector
%
% Output:
% A ... alpha*(mass matrix)+nu*(diffusion matrix)+convective matrix(Newton) 
%       for both velocity components u=[u1(1),u2(1),u1(2),u2(2),...]
% Bu, Bl ... divergence/gradient matrices
% E ... stabilization matrix
% b,c ... components of right-hand side vector
% a ... integral weights over domain
%
% Structure of the system: | A  Bu'| |u| = | b |
%                          | Bl -E | |p|   | c |
% with A = | A11 A12 |
%          | A21 A22 |

np=size(p,1); nt=size(t,1);
A=sparse(2*np,2*np); Bu=sparse(np,2*np); Bl=sparse(np,2*np); E=sparse(np,np);
b=zeros(2*np,1); c=zeros(np,1); a=zeros(np,1);
Mel=1/12*[2 1 1; 1 2 1; 1 1 2]; e=[1;1;1];
w1=w(1:2:end-1); w2=w(2:2:end);

for ih=1:nt
    itB=t(ih,1:3); it1=2*itB-1; it2=it1+1; it12=[it1,it2];  
    x21=p(t(ih,2),1)-p(t(ih,1),1); y12=p(t(ih,1),2)-p(t(ih,2),2);
    x32=p(t(ih,3),1)-p(t(ih,2),1); y23=p(t(ih,2),2)-p(t(ih,3),2);
    x13=p(t(ih,1),1)-p(t(ih,3),1); y31=p(t(ih,3),2)-p(t(ih,1),2);    
    x2D=[y23; y31; y12]; y2D=[x32; x13; x21];
    tarea=(x21*x2D(2)-x13*x2D(3))/2;
    
    % for convective matrix (Newton)
    w11=w1(t(ih,1)); w12=w1(t(ih,2)); w13=w1(t(ih,3));
    w21=w2(t(ih,1)); w22=w2(t(ih,2)); w23=w2(t(ih,3));
    w1t=(w11+w12+w13)/3; w2t=(w21+w22+w23)/3;
    dxw1=((w12-w11)*x2D(2)+(w13-w11)*x2D(3))/(2*tarea); 
    dyw1=((w12-w11)*y2D(2)+(w13-w11)*y2D(3))/(2*tarea);
    dxw2=((w22-w21)*x2D(2)+(w23-w21)*x2D(3))/(2*tarea); 
    dyw2=((w22-w21)*y2D(2)+(w23-w21)*y2D(3))/(2*tarea);
    
    mt1=3/20*tarea*e; mt=alpha*mt1;
    cl=9/40*(w1t*x2D+w2t*y2D);
    Zu=mt-cl; Zu=[Zu+dxw1*mt1 dyw1*mt1; dxw2*mt1 Zu+dyw2*mt1];
    Zl=mt+cl; Zl=[Zl+dxw1*mt1 dyw1*mt1; dxw2*mt1 Zl+dyw2*mt1];
        
    omegaM1=81*tarea/280;
    omega=81/40*nu*(x2D(1)^2+y2D(1)^2-x2D(2)*x2D(3)-y2D(2)*y2D(3))/tarea+alpha*omegaM1;
    
    omega11=omega+dxw1*omegaM1; omega12=dyw1*omegaM1; 
    omega21=dxw2*omegaM1; omega22=omega+dyw2*omegaM1;
    omega_det=omega11*omega22-omega12*omega21;
    invomega=1/omega_det*[omega22 -omega12; -omega21 omega11];
    
    % element matrices and vectors
    R=nu/4/tarea*(x2D*x2D'+y2D*y2D');
    M1=tarea*Mel;
    C=1/6*([w1t;w1t;w1t]*x2D'+[w2t;w2t;w2t]*y2D');
    Ah=[R+C+(alpha+dxw1)*M1 dyw1*M1
        dxw2*M1             R+C+(alpha+dyw2)*M1];
    Ah=Ah-Zu*invomega*Zl';
    
    Bh=1/6*[[x2D';x2D';x2D'],[y2D';y2D';y2D']]; 
    Bbh=9/40*[x2D,y2D]; 
    Buh=-Bh-Bbh*invomega'*Zu';
    Blh=-Bh-Bbh*invomega*Zl';
    Eh=Bbh*invomega*Bbh';
    
    f1t=(f(it1(1))+f(it1(2))+f(it1(3)))/3+dxw1*w1t+dyw1*w2t;         
    f2t=(f(it2(1))+f(it2(2))+f(it2(3)))/3+dxw2*w1t+dyw2*w2t;
    fh=tarea/3*[f1t;f1t;f1t;f2t;f2t;f2t];
    fbh=9/20*tarea*[f1t;f2t];                          
    ah=tarea/3*e;
    bh=fh-Zu*invomega*fbh;
    ch=-Bbh*invomega*fbh;

    % assembly global data
    A(it12,it12)=A(it12,it12)+Ah;
    Bu(itB,it12)=Bu(itB,it12)+Buh; 
    Bl(itB,it12)=Bl(itB,it12)+Blh;
    E(itB,itB)=E(itB,itB)+Eh;
    b(it12)=b(it12)+bh;
    c(itB)=c(itB)+ch; 
    a(itB)=a(itB)+ah;
end