function [A,Bu,Bl,E,b,c,a]=assemblyP1bP1CN_vec(p,t,nu,alpha,f,w)
% Navier-Stokes problem in 2D with P1-bubble/P1 elements for Newton linearization. 
% For Laplace operator. 
% Vectorized code.
% 
% Input:
% p ... fem nodes
% t ... fem triangles (counterclockwise oriented)
% alpha ... constant at mass matrix 
% nu ... viscosity constant
% f ... force at nodes
% w ... given velocity vector
%
% Output:
% A ... alpha*(mass matrix)+nu*(diffusion matrix)+convection matrix(Newton) 
%       for both velocity components u=[u1(1),u2(1),u1(2),u2(2),...]
% Bu, Bl ... divergence/gradient matrices
% E ... stabilization matrix
% b,c ... components of right-hand side vector
% a ... integral weights over domain
%
% Structure of the system: | A  Bu'| |u| = | b |
%                          | Bl -E | |p|   | c |
% with A = | A11 A12 |
%          | A21 A22 |

t_even=2*t; t_odd=t_even-1; tt=[t_odd;t_even];
np=size(p,1); np2=2*np;
A=sparse(np2,np2); Bu=sparse(np,np2); Bl=sparse(np,np2); E=sparse(np,np);
b=zeros(np2,1); c=zeros(np,1); a=zeros(np,1);
w1=w(1:2:end-1); w2=w(2:2:end);
x21=p(t(:,2),1)-p(t(:,1),1); y12=p(t(:,1),2)-p(t(:,2),2);
x32=p(t(:,3),1)-p(t(:,2),1); y23=p(t(:,2),2)-p(t(:,3),2);
x13=p(t(:,1),1)-p(t(:,3),1); y31=p(t(:,3),2)-p(t(:,1),2);
x2D=[y23 y31 y12]; y2D=[x32 x13 x21];
tarea=(x21.*x2D(:,2)-x13.*x2D(:,3))/2;

% for convective matrix (Newton)
w1t=(w1(t(:,1))+w1(t(:,2))+w1(t(:,3)))/3; 
w2t=(w2(t(:,1))+w2(t(:,2))+w2(t(:,3)))/3;
ttarea=2*tarea;
dxw1=((w1(t(:,2))-w1(t(:,1))).*y31+(w1(t(:,3))-w1(t(:,1))).*y12)./ttarea; 
dyw1=((w1(t(:,3))-w1(t(:,1))).*x21+(w1(t(:,2))-w1(t(:,1))).*x13)./ttarea;
dxw2=((w2(t(:,2))-w2(t(:,1))).*y31+(w2(t(:,3))-w2(t(:,1))).*y12)./ttarea; 
dyw2=((w2(t(:,3))-w2(t(:,1))).*x21+(w2(t(:,2))-w2(t(:,1))).*x13)./ttarea;
w1tx2Dw2ty2D=w1t.*x2D+w2t.*y2D;

omegaM1=81*tarea/280; 
omega=alpha*omegaM1+81/40*nu*(x2D(:,1).^2+y2D(:,1).^2-x2D(:,2).*x2D(:,3)-y2D(:,2).*y2D(:,3))./tarea;

omega11=omega+dxw1.*omegaM1; omega12=dyw1.*omegaM1; 
omega21=dxw2.*omegaM1;       omega22=omega+dyw2.*omegaM1;
omega_det=omega11.*omega22-omega12.*omega21;
invomega11= omega_det.\omega22; invomega12=-omega_det.\omega12;
invomega21=-omega_det.\omega21; invomega22= omega_det.\omega11;

% Mass matrix
tarea12=tarea/12; aux=alpha*tarea12;
for i=1:3
    for j=1:i
        A=A+sparse(tt(:,i),tt(:,j),[aux;aux],np2,np2);
    end
end
A=A+A';

% Diffusion matrix
n4ta=nu/4./tarea;
for i=1:3
    n4taxi=n4ta.*y2D(:,i); n4tayi=n4ta.*x2D(:,i);
    for j=1:i % A11, A22=A11 
        aux=n4tayi.*x2D(:,j)+n4taxi.*y2D(:,j);
        if i==j
            A=A+sparse(tt(:,i),tt(:,j),[aux;aux],np2,np2);
        else
            A=A+sparse(tt(:,i),tt(:,j),[aux;aux],np2,np2);
            A=A+sparse(tt(:,j),tt(:,i),[aux;aux],np2,np2);
        end
    end
end

% Convective matrix for the Oseen term
aux=1/6*w1tx2Dw2ty2D;
for i=1:3 
    for j=1:3
        A=A+sparse(tt(:,i),tt(:,j),[aux(:,j);aux(:,j)],np2,np2);
    end
end

% Convective matrix for the Newton term
aux1=dxw1.*tarea12; aux2=dyw2.*tarea12; aux3=dyw1.*tarea12; aux4=dxw2.*tarea12;
for i=1:3
    for j=1:i
        % A11, A22
        A=A+sparse(tt(:,i),tt(:,j),[aux1;aux2],np2,np2);
        A=A+sparse(tt(:,j),tt(:,i),[aux1;aux2],np2,np2);
        % A12, A21
        A=A+sparse(t_odd(:,i),t_even(:,j),aux3,np2,np2);
        A=A+sparse(t_odd(:,j),t_even(:,i),aux3,np2,np2);
        A=A+sparse(t_even(:,i),t_odd(:,j),aux4,np2,np2);
        A=A+sparse(t_even(:,j),t_odd(:,i),aux4,np2,np2);
    end
end

% Divergence matrix, Bu=B, Bl=B
for j=1:3
    aux=[x2D(:,j)/6;y2D(:,j)/6];
    for i=1:3 
        Bu=Bu-sparse([t(:,i);t(:,i)],tt(:,j),aux,np,np2);
    end
end
Bl=Bu;

% Bubble components elimination in A
mt1=3/20*tarea; mt=alpha*mt1; 
cl=9/40*w1tx2Dw2ty2D;
Zu=mt-cl; 
Zu11=Zu+dxw1.*mt1; Zu12=dyw1.*mt1; Zu12=[Zu12 Zu12 Zu12]; 
Zu21=dxw2.*mt1; Zu21=[Zu21 Zu21 Zu21]; Zu22=Zu+dyw2.*mt1;
Zl=mt+cl; 
Zl11=Zl+dxw1.*mt1; Zl12=dyw1.*mt1; Zl12=[Zl12 Zl12 Zl12]; 
Zl21=dxw2.*mt1; Zl21=[Zl21 Zl21 Zl21]; Zl22=Zl+dyw2.*mt1;
Zuio11=Zu11.*invomega11+Zu12.*invomega21; Zuio12=Zu11.*invomega12+Zu12.*invomega22;
Zuio21=Zu21.*invomega11+Zu22.*invomega21; Zuio22=Zu21.*invomega12+Zu22.*invomega22;

 for i=1:3
    for j=1:3
        aux1=Zuio11(:,i).*Zl11(:,j)+Zuio12(:,i).*Zl12(:,j);
        aux2=Zuio21(:,i).*Zl21(:,j)+Zuio22(:,i).*Zl22(:,j);
        aux3=Zuio11(:,i).*Zl21(:,j)+Zuio12(:,i).*Zl22(:,j);
        aux4=Zuio21(:,i).*Zl11(:,j)+Zuio22(:,i).*Zl12(:,j);
        A=A-sparse(tt(:,i),tt(:,j),[aux1;aux2],np2,np2);
        A=A-sparse(t_odd(:,i),t_even(:,j),aux3,np2,np2);
        A=A-sparse(t_even(:,i),t_odd(:,j),aux4,np2,np2);
    end
 end

% Bubble components elimination in Bu, Bl, E 
Bbh1=9/40*x2D; Bbh2=9/40*y2D;
Bbh1io=Bbh1.*invomega11+Bbh2.*invomega21; Bbh2io=Bbh1.*invomega12+Bbh2.*invomega22;
for i=1:3
    for j=1:3
        aux=Bbh1io(:,i).*Bbh1(:,j)+Bbh2io(:,i).*Bbh2(:,j);
        E=E+sparse(t(:,i),t(:,j),aux,np,np);
        aux1=Bbh1(:,i).*Zuio11(:,j)+Bbh2(:,i).*Zuio12(:,j);
        aux2=Bbh1(:,i).*Zuio21(:,j)+Bbh2(:,i).*Zuio22(:,j);
        Bu=Bu-sparse([t(:,i);t(:,i)],tt(:,j),[aux1;aux2],np,np2);
        aux1=Bbh1io(:,i).*Zl11(:,j)+Bbh2io(:,i).*Zl12(:,j);
        aux2=Bbh1io(:,i).*Zl21(:,j)+Bbh2io(:,i).*Zl22(:,j);
        Bl=Bl-sparse([t(:,i);t(:,i)],tt(:,j),[aux1;aux2],np,np2);
    end
end

% Right hand-side vector b
f1=f(1:2:end-1); f2=f(2:2:end);
f1=1/3*tarea.*((f1(t(:,1))+f1(t(:,2))+f1(t(:,3)))/3+dxw1.*w1t+dyw1.*w2t); 
f2=1/3*tarea.*((f2(t(:,1))+f2(t(:,2))+f2(t(:,3)))/3+dxw2.*w1t+dyw2.*w2t);
for i=1:3
    b=b+sparse(tt(:,i),1,[f1;f2],np2,1);
end

% Bubble components elimination in b, c and a
f1b=27/20.*f1; f2b=27/20.*f2; 
for i=1:3
    aux1=Zuio11(:,i).*f1b+Zuio12(:,i).*f2b;
    aux2=Zuio21(:,i).*f1b+Zuio22(:,i).*f2b;
    b=b-sparse(tt(:,i),1,[aux1;aux2],np2,1);
    aux=Bbh1io(:,i).*f1b+Bbh2io(:,i).*f2b;
    c=c-sparse(t(:,i),1,aux,np,1);
    a=a+sparse(t(:,i),1,tarea/3,np,1);
end