function [A,Bu,Bl,E,b,c,a]=assemblyP1bP1DCN_vecO(p,t,nu,alpha,f,w)
% Navier-Stokes problem in 2D with P1-bubble/P1 elements for Newton linearization. 
% For symmetric part of gradient.
% Vectorized code.
% Optimized version.
%
% Input:
% p ... fem nodes
% t ... fem triangles (counterclockwise oriented)
% alpha ... constant at mass matrix 
% nu ... viscosity constant
% f ... force at nodes
% w ... given velocity vector
%
% Output:
% A ... alpha*(mass matrix)+nu*(diffusion matrix)+convection matrix(Newton) 
%       for both velocity components u=[u1(1),u2(1),u1(2),u2(2),...]
% Bu, Bl ... divergence/gradient matrices
% E ... stabilization matrix
% b,c ... components of right-hand side vector
% a ... integral weights over domain
%
% Structure of the system: | A  Bu'| |u| = | b |
%                          | Bl -E | |p|   | c |
% with A = | A11 A12 |
%          | A21 A22 |
np=size(p,1);
% 1. triangles area
x32=p(t(:,3),1)-p(t(:,2),1); y23=p(t(:,2),2)-p(t(:,3),2);
x13=p(t(:,1),1)-p(t(:,3),1); y31=p(t(:,3),2)-p(t(:,1),2);
x21=p(t(:,2),1)-p(t(:,1),1); y12=p(t(:,1),2)-p(t(:,2),2);
tarea=(x21.*y31-x13.*y12)/2;
% 2. x^(T), y^(T), z^(T) and omega
xt=[y23 y31 y12]; yt=[x32 x13 x21];

w1=w(1:2:end-1); w2=w(2:2:end);
w1t=(w1(t(:,1))+w1(t(:,2))+w1(t(:,3)))/3;
w2t=(w2(t(:,1))+w2(t(:,2))+w2(t(:,3)))/3;

cal=(1/6)*[w1t.*xt(:,1)+w2t.*yt(:,1),w1t.*xt(:,2)+w2t.*yt(:,2),w1t.*xt(:,3)+w2t.*yt(:,3)];

w1x=(1/2)*((w1(t(:,2))-w1(t(:,1))).*y31+(w1(t(:,3))-w1(t(:,1))).*y12);
w1y=(1/2)*((w1(t(:,3))-w1(t(:,1))).*x21+(w1(t(:,2))-w1(t(:,1))).*x13);
w2x=(1/2)*((w2(t(:,2))-w2(t(:,1))).*y31+(w2(t(:,3))-w2(t(:,1))).*y12);
w2y=(1/2)*((w2(t(:,3))-w2(t(:,1))).*x21+(w2(t(:,2))-w2(t(:,1))).*x13);

nut=nu./tarea;
omega_11 = (81/280)*(alpha*tarea+w1x)+(81/20)*nut.*(y23.^2-y31.*y12+(1/2)*(x32.^2-x13.*x21));
omega_12 = (81/280)*w1y+(81/80)*nut.*(y23.*x32+y31.*x13+y12.*x21);
omega_21 = omega_12+(81/280)*(w2x-w1y);
omega_22 = (81/280)*(alpha*tarea+w2y)+(81/20)*nut.*((1/2)*(y23.^2-y31.*y12)+x32.^2-x13.*x21);
invomega_N = 1./(omega_11.*omega_22-omega_12.*omega_21);

am_11=omega_11.*invomega_N;
am_12=omega_12.*invomega_N;
am_21=omega_21.*invomega_N;
am_22=omega_22.*invomega_N;

zu = ((3*alpha)/20)*tarea-(9/40)*(w1t.*xt+w2t.*yt);
zl = ((3*alpha)/10)*tarea-zu;

zu_11 = zu + (3/20)*w1x;
zu_22 = zu + (3/20)*w2y;
zl_11 = zl + (3/20)*w1x;
zl_22 = zl + (3/20)*w2y;

z_12 = (3/20)*w1y;
z_21 = (3/20)*w2x;

f1=f(1:2:end-1); f2=f(2:2:end);
f1t=(1/3)*(tarea.*(f1(t(:,1))+f1(t(:,2))+f1(t(:,3)))/3 + w1x.*w1t + w1y.*w2t);
f2t=(1/3)*(tarea.*(f2(t(:,1))+f2(t(:,2))+f2(t(:,3)))/3 + w2x.*w1t + w2y.*w2t);

f1b=(27/20)*(am_22.*f1t-am_12.*f2t);
f2b=(27/20)*(am_11.*f2t-am_21.*f1t);

w1x=(alpha/12)*tarea+(1/12)*w1x;
w1y=(1/12)*w1y;
w2x=(1/12)*w2x;
w2y=(alpha/12)*tarea+(1/12)*w2y;

A11 = sparse(np,np); A22 = sparse(np,np);
A12 = sparse(np,np); A21 = sparse(np,np);
Bu1 = sparse(np,np); Bu2 = sparse(np,np);
Bl1 = sparse(np,np); Bl2 = sparse(np,np);
E = sparse(np,np);
b1 = zeros(np,1); b2 = zeros(np,1);
c = zeros(np,1); a=zeros(np,1);
for i=1:3
    a=a+sparse(t(:,i),1,tarea/3,np,1);
    b1=b1 + sparse(t(:,i),1,f1t-zu_11(:,i).*f1b-z_12.*f2b,np,1);
    b2=b2 + sparse(t(:,i),1,f2t-z_21.*f1b-zu_22(:,i).*f2b,np,1);
    c=c-sparse(t(:,i),1,(9/40)*(xt(:,i).*f1b+yt(:,i).*f2b),np,1);

    l11=(am_21.*z_12-am_22.*zu_11(:,i));
    l12=(am_12.*zu_11(:,i)-am_11.*z_12);
    l21=(am_21.*zu_22(:,i)-am_22.*z_21);
    l22=(am_12.*z_21-am_11.*zu_22(:,i));

    B1=(9/40)*(am_22.*xt(:,i)-am_12.*yt(:,i));
    B2=(9/40)*(am_11.*yt(:,i)-am_21.*xt(:,i));
    E1=(9/40)*(am_22.*xt(:,i)-am_21.*yt(:,i));
    E2=(9/40)*(am_11.*yt(:,i)-am_12.*xt(:,i));
    for j=1:3        
        if(i==j)
            A11=A11+sparse(t(:,i),t(:,j),2*w1x+cal(:,j)+(nut/4).*(yt(:,i).*yt(:,j)+2*xt(:,i).*xt(:,j))+(l11.*zl_11(:,j)+l12.*z_12),np,np);
            A12=A12+sparse(t(:,i),t(:,j),2*w1y+(nut/4).*yt(:,i).*xt(:,j)+(l11.*z_21+l12.*zl_22(:,j)),np,np);
            A21=A21+sparse(t(:,i),t(:,j),2*w2x+(nut/4).*xt(:,i).*yt(:,j)+(l21.*zl_11(:,j)+z_12.*l22),np,np);
            A22=A22+sparse(t(:,i),t(:,j),2*w2y+cal(:,j)+(nut/4).*(2*yt(:,i).*yt(:,j)+xt(:,i).*xt(:,j))+(l21.*z_21+l22.*zl_22(:,j)),np,np); 
        else
            A11=A11+sparse(t(:,i),t(:,j),w1x+cal(:,j)+(nut/4).*(yt(:,i).*yt(:,j)+2*xt(:,i).*xt(:,j))+(l11.*zl_11(:,j)+l12.*z_12),np,np);
            A12=A12+sparse(t(:,i),t(:,j),w1y+(nut/4).*yt(:,i).*xt(:,j)+(l11.*z_21+l12.*zl_22(:,j)),np,np);
            A21=A21+sparse(t(:,i),t(:,j),w2x+(nut/4).*xt(:,i).*yt(:,j)+(l21.*zl_11(:,j)+z_12.*l22),np,np);
            A22=A22+sparse(t(:,i),t(:,j),w2y+cal(:,j)+(nut/4).*(2*yt(:,i).*yt(:,j)+xt(:,i).*xt(:,j))+(l21.*z_21+l22.*zl_22(:,j)),np,np); 
        end
        Bu1=Bu1-sparse(t(:,i),t(:,j),xt(:,j)/6+(zu_11(:,j).*B1+z_12.*B2),np,np);
        Bu2=Bu2-sparse(t(:,i),t(:,j),yt(:,j)/6+(z_21.*B1+zu_22(:,j).*B2),np,np);
        Bl1=Bl1-sparse(t(:,i),t(:,j),xt(:,j)/6+(zl_11(:,j).*E1+z_12.*E2),np,np);
        Bl2=Bl2-sparse(t(:,i),t(:,j),yt(:,j)/6+(z_21.*E1+zl_22(:,j).*E2),np,np);
        E=E+sparse(t(:,i),t(:,j),(9/40)*(xt(:,j).*E1+yt(:,j).*E2),np,np);
    end
end
t2(1:2:2*np-1)=1:np; t2(2:2:2*np)=np+(1:np);
A=[A11 A12;A21 A22]; A=A(t2,t2);
Bu=[Bu1 Bu2]; Bu=Bu(:,t2);
Bl=[Bl1 Bl2]; Bl=Bl(:,t2);
b=[b1;b2]; b=b(t2,1);
end