function [A,Bu,Bl,E,b,c,a]=assemblyP1bP1C_vec(p,t,nu,alpha,f,w)
% Navier-Stokes problem in 2D with P1-bubble/P1 elements for Oseen linearization. 
% For Laplace operator. 
% Vectorized code.
% 
% Input:
% p ... fem nodes
% t ... fem triangles (counterclockwise oriented)
% alpha ... constant at mass matrix 
% nu ... viscosity constant
% f ... force at nodes
% w ... given velocity vector
%
% A ... alpha*(mass matrix)+nu*(diffusion matrix)+convection matrix(Oseen) 
%       for both velocity components u=[u1(1),u2(1),u1(2),u2(2),...]
% Bu, Bl ... divergence/gradient matrices
% E ... stabilization matrix
% b,c ... components of right-hand side vector
% a ... integral weights over domain
%
% Structure of the system: | A  Bu'| |u| = | b |
%                          | Bl -E | |p|   | c |
% with A = | A11 0   |
%          | 0   A22 |
% A11=A22 being nonsymmetric 

tt=2*t; tt=[tt-1;tt];
np=size(p,1); np2=2*np;
A=sparse(np2,np2); Bu=sparse(np,2*np); Bl=sparse(np,2*np); E=sparse(np,np);
b=zeros(np2,1); c=zeros(np,1); a=zeros(np,1);
w1=w(1:2:end-1); w2=w(2:2:end);
x21=p(t(:,2),1)-p(t(:,1),1); y12=p(t(:,1),2)-p(t(:,2),2);
x32=p(t(:,3),1)-p(t(:,2),1); y23=p(t(:,2),2)-p(t(:,3),2);
x13=p(t(:,1),1)-p(t(:,3),1); y31=p(t(:,3),2)-p(t(:,1),2);
x2D=[y23 y31 y12]; y2D=[x32 x13 x21];
tarea=(x21.*x2D(:,2)-x13.*x2D(:,3))/2; alta=alpha*tarea;

% for convective matrix
w1t=(w1(t(:,1))+w1(t(:,2))+w1(t(:,3)))/3; 
w2t=(w2(t(:,1))+w2(t(:,2))+w2(t(:,3)))/3;
w1tx2Dw2ty2D=w1t.*x2D+w2t.*y2D;

omega=81/40*nu*(x2D(:,1).^2+y2D(:,1).^2-x2D(:,2).*x2D(:,3)-y2D(:,2).*y2D(:,3))./tarea+81/280*alta;

% Mass matrix
aux=alta/12;
for i=1:3
    for j=1:i
        A=A+sparse(tt(:,i),tt(:,j),[aux;aux],np2,np2);
    end
end
A=A+A';

% Diffusion matrix
n4ta=nu/4./tarea;
for i=1:3
    n4taxi=n4ta.*y2D(:,i); n4tayi=n4ta.*x2D(:,i);
    for j=1:i % A11, A22=A11 
        aux=n4tayi.*x2D(:,j)+n4taxi.*y2D(:,j);
        if i==j
            A=A+sparse(tt(:,i),tt(:,j),[aux;aux],np2,np2);
        else
            A=A+sparse(tt(:,i),tt(:,j),[aux;aux],np2,np2);
            A=A+sparse(tt(:,j),tt(:,i),[aux;aux],np2,np2);
        end
    end
end

% Convective matrix for the Oseen term
aux=1/6*w1tx2Dw2ty2D;
for i=1:3 
    for j=1:3
        A=A+sparse(tt(:,i),tt(:,j),[aux(:,j);aux(:,j)],np2,np2);
    end
end

% Divergence matrix, Bu=B, Bl=B
for j=1:3
    aux=[x2D(:,j)/6;y2D(:,j)/6];
    for i=1:3 
        Bu=Bu-sparse([t(:,i);t(:,i)],tt(:,j),aux,np,np2);
    end
end
Bl=Bu;

% Bubble components elimination in A
mt=3/20*alta; clt=9/40*w1tx2Dw2ty2D;
Zu=mt-clt; Zl=mt+clt; Zuiom=Zu./omega;
for i=1:3
    for j=1:3
        aux=Zuiom(:,i).*Zl(:,j);
        A=A-sparse(tt(:,i),tt(:,j),[aux;aux],np2,np2);
    end
end

% Bubble components elimination in Bu, Bl, E
Bbh1=9/40*x2D; Bbh1iom=Bbh1./omega;
Bbh2=9/40*y2D; Bbh2iom=Bbh2./omega;
for i=1:3
    for j=1:3
        aux=Bbh1iom(:,i).*Bbh1(:,j)+Bbh2iom(:,i).*Bbh2(:,j);
        E=E+sparse(t(:,i),t(:,j),aux,np,np);
        aux1=Bbh1iom(:,i).*Zu(:,j); aux2=Bbh2iom(:,i).*Zu(:,j);
        Bu=Bu-sparse([t(:,i);t(:,i)],tt(:,j),[aux1;aux2],np,np2);
        aux1=Bbh1iom(:,i).*Zl(:,j); aux2=Bbh2iom(:,i).*Zl(:,j);
        Bl=Bl-sparse([t(:,i);t(:,i)],tt(:,j),[aux1;aux2],np,np2);
     end
end

% Right hand-side vector b
f1=f(1:2:end-1); f2=f(2:2:end);
f1=tarea.*(f1(t(:,1))+f1(t(:,2))+f1(t(:,3)))/9 ; 
f2=tarea.*(f2(t(:,1))+f2(t(:,2))+f2(t(:,3)))/9;
f1b=27/20*f1; f2b=27/20*f2;
for i=1:3
    aux1=f1-Zuiom(:,i).*f1b; aux2=f2-Zuiom(:,i).*f2b;
    b=b+sparse(tt(:,i),1,[aux1;aux2],np2,1);
end

% c and a
for i=1:3
    aux=Bbh1iom(:,i).*f1b+Bbh2iom(:,i).*f2b;
    c=c-sparse(t(:,i),1,aux,np,1);
    a=a+sparse(t(:,i),1,tarea/3,np,1);
end