function [A,Bu,Bl,E,b,c,a] = assemblyP1bP1C_vecO(p,t,nu,alpha,f,w)
% Navier-Stokes problem in 2D with P1-bubble/P1 elements for Oseen linearization.
% For Laplace operator.
% Vectorized code.
% Optimized version.
%
% Input:
% p ... fem nodes
% t ... fem triangles (counterclockwise oriented)
% alpha ... constant at mass matrix
% nu ... viscosity constant
% f ... force at nodes
% w ... given velocity vector
%
% A ... alpha*(mass matrix)+nu*(diffusion matrix)+convection matrix(Oseen)
%       for both velocity components u=[u1(1),u2(1),u1(2),u2(2),...]
% Bu, Bl ... divergence/gradient matrices
% E ... stabilization matrix
% b,c ... components of right-hand side vector
% a ... integral weights over domain
%
% Structure of the system: | A  Bu'| |u| = | b |
%                          | Bl -E | |p|   | c |
% with A = | A11 0   |
%          | 0   A22 |
% A11=A22 being nonsymmetric
np=size(p,1);
% 1. triangles area
x21=p(t(:,2),1)-p(t(:,1),1); y12=p(t(:,1),2)-p(t(:,2),2);
x32=p(t(:,3),1)-p(t(:,2),1); y23=p(t(:,2),2)-p(t(:,3),2);
x13=p(t(:,1),1)-p(t(:,3),1); y31=p(t(:,3),2)-p(t(:,1),2);
tarea=(x21.*y31-x13.*y12)/2;
% 2. x^(T), y^(T), z^(T) and omega
xt=[y23 y31 y12]; yt=[x32 x13 x21];

nut=nu./tarea;
invomega=280./(81*(alpha*tarea+(7*nut).*(y23.^2+x32.^2-y31.*y12-x13.*x21)));

w1=w(1:2:end-1); w2=w(2:2:end);
w1t=(w1(t(:,1))+w1(t(:,2))+w1(t(:,3)))/3;
w2t=(w2(t(:,1))+w2(t(:,2))+w2(t(:,3)))/3;
cal=(1/6)*[w1t.*xt(:,1)+w2t.*yt(:,1),w1t.*xt(:,2)+w2t.*yt(:,2),w1t.*xt(:,3)+w2t.*yt(:,3)];


zu =((3*alpha)/20)*tarea-(9/40)*(w1t.*xt+w2t.*yt);
zl =((3*alpha)/10)*tarea-zu;


f1=f(1:2:end-1);f2=f(2:2:end);
f1t=tarea.*(f1(t(:,1))+f1(t(:,2))+f1(t(:,3)))/9;
f2t=tarea.*(f2(t(:,1))+f2(t(:,2))+f2(t(:,3)))/9;

f1b=(27/20)*f1t.*invomega;
f2b=(27/20)*f2t.*invomega;

Ah=sparse(np,np);  E=sparse(np,np);
Bu1=sparse(np,np); Bu2=sparse(np,np);
Bl1=sparse(np,np); Bl2=sparse(np,np);
b1=zeros(np,1); b2=zeros(np,1);
c=zeros(np,1); a=zeros(np,1);
for i=1:3
    a=a+sparse(t(:,i),1,tarea/3,np,1);
    b1=b1+sparse(t(:,i),1,f1t-zu(:,i).*f1b,np,1);
    b2=b2+sparse(t(:,i),1,f2t-zu(:,i).*f2b,np,1);
    c=c-sparse(t(:,i),1,(9/40)*(xt(:,i).*f1b+yt(:,i).*f2b),np,1);

    E1=(9/40)*xt(:,i).*invomega;
    E2=(9/40)*yt(:,i).*invomega;

    for j=1:3
        if(i==j)
            Ah=Ah+sparse(t(:,i),t(:,j),(alpha/6)*tarea+(nut/4).*(xt(:,i).*xt(:,j)+yt(:,i).*yt(:,j))+cal(:,j)-zl(:,j).*zu(:,i).*invomega,np,np);
        else
            Ah=Ah+sparse(t(:,i),t(:,j),(alpha/12)*tarea+(nut/4).*(xt(:,i).*xt(:,j)+yt(:,i).*yt(:,j))+cal(:,j)-zl(:,j).*zu(:,i).*invomega,np,np);
        end
        Bu1=Bu1-sparse(t(:,i),t(:,j),xt(:,j)/6+zu(:,j).*E1,np,np);
        Bu2=Bu2-sparse(t(:,i),t(:,j),yt(:,j)/6+zu(:,j).*E2,np,np);
        Bl1=Bl1-sparse(t(:,i),t(:,j),xt(:,j)/6+zl(:,j).*E1,np,np);
        Bl2=Bl2-sparse(t(:,i),t(:,j),yt(:,j)/6+zl(:,j).*E2,np,np);
        E=E+sparse(t(:,i),t(:,j),(9/40)*(E1.*xt(:,j)+E2.*yt(:,j)),np,np);
    end
end
t2(1:2:2*np-1)=1:np; t2(2:2:2*np)=np+(1:np);
A=[Ah sparse(np,np);sparse(np,np) Ah]; A=A(t2,t2);
Bu=[Bu1 Bu2]; Bu=Bu(:,t2);
Bl=[Bl1 Bl2]; Bl=Bl(:,t2);
b=[b1;b2]; b=b(t2,1);
end