function [A,Bu,Bl,E,b,c,a]=assemblyP1bP1DC(p,t,nu,alpha,f,w)
% Navier-Stokes problem in 2D with P1-bubble/P1 elements for Oseen linearization. 
% For symmetric part of gradient.
% Loop over triangles.
%
% Input:
% p ... fem nodes
% t ... fem triangles (counterclockwise oriented)
% alpha ... constant at mass matrix 
% nu ... viscosity constant
% f ... force at nodes
% w ... given velocity vector
%
% Output:
% A ... alpha*(mass matrix)+nu*(diffusion matrix)+convective matrix(Oseen) 
%       for both velocity components u=[u1(1),u2(1),u1(2),u2(2),...]
% Bu, Bl ... divergence/gradient matrices
% E ... stabilization matrix
% b,c ... components of right-hand side vector
% a ... integral weights over domain
%
% Structure of the system: | A  Bu'| |u| = | b |
%                          | Bl -E | |p|   | c |
% with A = | A11 A12 |
%          | A21 A22 |
% A11=A22 being nonsymmetric, A21=A12'

np=size(p,1); nt=size(t,1);
A=sparse(2*np,2*np); Bu=sparse(np,2*np); Bl=sparse(np,2*np); E=sparse(np,np);
b=zeros(2*np,1); c=zeros(np,1); a=zeros(np,1);
Mel=1/12*[2 1 1; 1 2 1; 1 1 2]; e=[1;1;1];
w1=w(1:2:end-1); w2=w(2:2:end);

for ih=1:nt
    itB=t(ih,1:3); it1=2*itB-1; it2=it1+1; it12=[it1,it2]; 
    x21=p(t(ih,2),1)-p(t(ih,1),1); y12=p(t(ih,1),2)-p(t(ih,2),2);
    x32=p(t(ih,3),1)-p(t(ih,2),1); y23=p(t(ih,2),2)-p(t(ih,3),2);
    x13=p(t(ih,1),1)-p(t(ih,3),1); y31=p(t(ih,3),2)-p(t(ih,1),2);    
    x2D=[y23; y31; y12]; y2D=[x32; x13; x21]; 
    tarea=(x21*x2D(2)-x13*x2D(3))/2;
    
    % for convective matrix (Oseen)
    w1t=(w1(t(ih,1))+w1(t(ih,2))+w1(t(ih,3)))/3;
    w2t=(w2(t(ih,1))+w2(t(ih,2))+w2(t(ih,3)))/3;
    
    mt=3/20*alpha*tarea*e; cl=9/40*(w1t*x2D+w2t*y2D);
    Zu=mt-cl; Zl=mt+cl;
    Zu=[Zu,zeros(3,1);zeros(3,1),Zu];
    Zl=[Zl,zeros(3,1);zeros(3,1),Zl];

    omega11=81/20*nu*(x2D(1)^2+y2D(1)^2/2-x2D(2)*x2D(3)-y2D(2)*y2D(3)/2)/tarea+81/280*alpha*tarea;
    omega22=81/20*nu*(x2D(1)^2/2+y2D(1)^2-x2D(2)*x2D(3)/2-y2D(2)*y2D(3))/tarea+81/280*alpha*tarea;
    omega12=81*nu/80/tarea*y2D'*x2D; omega21=omega12;
    omega_det=omega11*omega22-omega12^2;
    invomega=[omega22/omega_det -omega12/omega_det; -omega12/omega_det omega11/omega_det];
    
    % element matrices
    M=alpha*tarea*Mel;
    C=1/6*(w1t*[x2D';x2D';x2D']+w2t*[y2D';y2D';y2D']);
    Ah=[C+M+nu/2/tarea*(x2D*x2D'+y2D*y2D'/2) nu/4/tarea*y2D*x2D'
        nu/4/tarea*x2D*y2D'                  C+M+nu/2/tarea*(x2D*x2D'/2+y2D*y2D')];
    Ah=Ah-Zu*invomega*Zl';
    Bh =-1/6*[[x2D'; x2D'; x2D'],[y2D'; y2D'; y2D']];
    Bbh=9/40*[x2D,y2D];
    Bhl=Bh-Bbh*invomega*Zl';
    Bhu=Bh-Bbh*invomega*Zu';
    Eh=Bbh*invomega*Bbh';
    
    % element vectors
    f1t=(f(it1(1))+f(it1(2))+f(it1(3)))/3; f2t=(f(it2(1))+f(it2(2))+f(it2(3)))/3;
    fh=tarea/3*[f1t;f1t;f1t;f2t;f2t;f2t];  fbh=9/20*tarea*[f1t;f2t];
    bh=fh-Zu*invomega*fbh;               
    ch=-Bbh*invomega*fbh;
    ah= tarea/3*e;

    % assembly global data
    A(it12,it12)=A(it12,it12)+Ah;
    Bu(itB,it12)=Bu(itB,it12)+Bhu;
    Bl(itB,it12)=Bl(itB,it12)+Bhl;
    E(itB,itB)=E(itB,itB)+Eh;

    b(it12)=b(it12)+bh;
    c(itB)=c(itB)+ch; a(itB)=a(itB)+ah;
end