function [A,Bu,Bl,E,b,c,a]=assemblyP1bP1DC_vec(p,t,nu,alpha,f,w)
% Navier-Stokes problem in 2D with P1-bubble/P1 elements for Oseen linearization. 
% For symmetric part of gradient.
% Vectorized code.
% 
% Input:
% p ... fem nodes
% t ... fem triangles (counterclockwise oriented)
% alpha ... constant at mass matrix 
% nu ... viscosity constant
% f ... force at nodes
% w ... given velocity vector
%
% Output:
% A ... alpha*(mass matrix)+nu*(diffusion matrix)+convective matrix(Oseen) 
%       for both velocity components u=[u1(1),u2(1),u1(2),u2(2),...]
% Bu, Bl ... divergence/gradient matrices
% E ... stabilization matrix
% b,c ... components of right-hand side vector
% a ... integral weights over domain
%
% Structure of the system: | A  Bu'| |u| = | b |
%                          | Bl -E | |p|   | c |
% with A = | A11 A12 |
%          | A21 A22 |
% A11=A22 being nonsymmetric, A21=A12'

t_even=2*t; t_odd=t_even-1; tt=[t_odd;t_even];
np=size(p,1); np2=2*np;
A=sparse(np2,np2); Bu=sparse(np,np2); Bl=sparse(np,np2); E=sparse(np,np);
b=zeros(np2,1); c=zeros(np,1); a=zeros(np,1);
w1=w(1:2:end-1); w2=w(2:2:end);
x21=p(t(:,2),1)-p(t(:,1),1); y12=p(t(:,1),2)-p(t(:,2),2);
x32=p(t(:,3),1)-p(t(:,2),1); y23=p(t(:,2),2)-p(t(:,3),2);
x13=p(t(:,1),1)-p(t(:,3),1); y31=p(t(:,3),2)-p(t(:,1),2);
x2D=[y23 y31 y12]; y2D=[x32 x13 x21];
tarea=(x21.*x2D(:,2)-x13.*x2D(:,3))/2; alta=alpha*tarea;

% for convective matrix
w1t=(w1(t(:,1))+w1(t(:,2))+w1(t(:,3)))/3; 
w2t=(w2(t(:,1))+w2(t(:,2))+w2(t(:,3)))/3;
w1tx2Dw2ty2D=w1t.*x2D+w2t.*y2D; 

mt=3/20*alta; cl=9/40*w1tx2Dw2ty2D;
Zu=mt-cl; Zl=mt+cl;

x2D12=x2D(:,1).^2; y2D12=y2D(:,1).^2; x2D2x2D3=x2D(:,2).*x2D(:,3); y2D2y2D3=y2D(:,2).*y2D(:,3);
omega11=81*nu/20*(x2D12+y2D12/2-x2D2x2D3-y2D2y2D3/2)./tarea+81/280*alta;
omega22=81*nu/20*(x2D12/2+y2D12-x2D2x2D3/2-y2D2y2D3)./tarea+81/280*alta;
omega12=81/80*nu*(x2D(:,1).*y2D(:,1)+x2D(:,2).*y2D(:,2)+x2D(:,3).*y2D(:,3))./tarea;
omega_det=omega11.*omega22-omega12.^2;
invomega11=omega22./omega_det; 
invomega22=omega11./omega_det;
invomega12=-omega12./omega_det;

% Mass matrix
aux=alta/12;
for i=1:3
    for j=1:i
        A=A+sparse(tt(:,i),tt(:,j),[aux;aux],np2,np2);
    end
end
A=A+A';

% Diffusion matrix
n4ta=nu/4./tarea;
for i=1:3
    n4taxi=n4ta.*y2D(:,i); n4tayi=n4ta.*x2D(:,i);
    for j=i:3 % A11, A22 
        n4taxixj=n4taxi.*y2D(:,j); n4tayiyj=n4tayi.*x2D(:,j);
        aux1=2*n4tayiyj+n4taxixj;
        aux2=n4tayiyj+2*n4taxixj;
        if i==j
            A=A+sparse(tt(:,i),tt(:,j),[aux1;aux2],np2,np2);
        else
            A=A+sparse(tt(:,i),tt(:,j),[aux1;aux2],np2,np2);
            A=A+sparse(tt(:,j),tt(:,i),[aux1;aux2],np2,np2);
        end
    end
    for j=1:3 % A12, A21
        aux=n4taxi.*x2D(:,j);
        A=A+sparse(t_odd(:,i),t_even(:,j),aux,np2,np2);
        A=A+sparse(t_even(:,j),t_odd(:,i),aux,np2,np2);
    end
end

% Convection matrix for the Oseen term
aux=1/6*w1tx2Dw2ty2D;
for i=1:3 
    for j=1:3
        A=A+sparse(tt(:,i),tt(:,j),[aux(:,j);aux(:,j)],np2,np2);
    end
end

% Divergence matrix, Bu=B, Bl=B
for j=1:3
    aux=[x2D(:,j)/6;y2D(:,j)/6];
    for i=1:3 
        Bu=Bu-sparse([t(:,i);t(:,i)],tt(:,j),aux,np,np2);
    end
end
Bl=Bu;

% Bubble components elimination in A
for i=1:3
    for j=1:3
        aux=Zu(:,i).*Zl(:,j);
        A=A-sparse(tt(:,i),tt(:,j),[aux.*invomega11;aux.*invomega22],np2,np2);
        A=A-sparse(t_odd(:,i),t_even(:,j),aux.*invomega12,np2,np2);
        A=A-sparse(t_even(:,i),t_odd(:,j),aux.*invomega12,np2,np2);
    end
end

% Bubble components elimination in Bu, Bl, E 
B1b=9/40*x2D; B2b=9/40*y2D; 
aux1=B1b.*invomega11+B2b.*invomega12; 
aux2=B1b.*invomega12+B2b.*invomega22;
for i=1:3
    for j=1:3
        E=E+sparse(t(:,i),t(:,j),aux1(:,i).*B1b(:,j)+aux2(:,i).*B2b(:,j),np,np);
        Bu=Bu-sparse([t(:,i);t(:,i)],tt(:,j),[aux1(:,i).*Zu(:,j);aux2(:,i).*Zu(:,j)],np,np2);
        Bl=Bl-sparse([t(:,i);t(:,i)],tt(:,j),[aux1(:,i).*Zl(:,j);aux2(:,i).*Zl(:,j)],np,np2);
    end
end

% Right hand-side vector b
f1=f(1:2:np2); f2=f(2:2:np2);
f1=tarea.*(f1(t(:,1))+f1(t(:,2))+f1(t(:,3)))/9 ; 
f2=tarea.*(f2(t(:,1))+f2(t(:,2))+f2(t(:,3)))/9;
for i=1:3
    b=b+sparse(tt(:,i),1,[f1;f2],np2,1);
end

% Bubble components elimination in b, c and a
f1b=27/20.*f1; f2b=27/20.*f2; 
aux1=invomega11.*f1b+invomega12.*f2b; 
aux2=invomega12.*f1b+invomega22.*f2b;
for i=1:3
    b=b-sparse(tt(:,i),1,[Zu(:,i).*aux1;Zu(:,i).*aux2],np2,1);
    c=c-sparse(t(:,i),1,B1b(:,i).*aux1+B2b(:,i).*aux2,np,1);
    a=a+sparse(t(:,i),1,tarea/3,np,1);
end