function [A,B,E,b,c,a]=assemblyP1bP1(p,t,nu,alpha,f)
% Stokes problem in 2D with P1-bubble/P1 elements.
% For Laplace operator.
% Loop over triangles.
%
% Input:
% p ... fem nodes
% t ... fem triangles (counterclockwise oriented)
% alpha ... constant at mass matrix 
% nu ... viscosity constant
% f ... force at nodes
%
% Output:
% A ... alpha*(mass matrix)+nu*(diffusion matrix) 
%       for both velocity components u=[u1(1),u2(1),u1(2),u2(2),...]
% B ... divergence matrix
% E ... stabilization matrix
% b,c ... components of right-hand side vector
% a ... integral weights over domain
%
% Structure of the system: | A  B'| |u| = | b |
%                          | B -E | |p|   | c |
% with A = | A11 0   |
%          | 0   A22 |
% A11=A22 being symmetric

np=size(p,1); nt=size(t,1);
A=sparse(2*np,2*np); B=sparse(np,2*np); E=sparse(np,np);
b=zeros(2*np,1); c=zeros(np,1); a=zeros(np,1);
Mel=1/12*[2 1 1; 1 2 1; 1 1 2]; e=[1;1;1];

for ih=1:nt
    itB=t(ih,1:3); it1=2*itB-1; it2=it1+1; 
    x21=p(t(ih,2),1)-p(t(ih,1),1); y12=p(t(ih,1),2)-p(t(ih,2),2);
    x32=p(t(ih,3),1)-p(t(ih,2),1); y23=p(t(ih,2),2)-p(t(ih,3),2);
    x13=p(t(ih,1),1)-p(t(ih,3),1); y31=p(t(ih,3),2)-p(t(ih,1),2);    
    x2D=[y23; y31; y12]; y2D=[x32; x13; x21]; 
    tarea=(x21*x2D(2)-x13*x2D(3))/2;
    Z=3/20*alpha*tarea*e;
    omega=81/40*nu*(x2D(1)^2+y2D(1)^2-x2D(2)*x2D(3)-y2D(2)*y2D(3))/tarea+81/280*alpha*tarea;

    % element matrices and vectors
    Ah=alpha*tarea*Mel+nu*1/4/tarea*(y2D*y2D'+x2D*x2D')-1/omega*Z*Z';
    B1h=-1/6*[x2D';x2D';x2D']-9/40/omega*x2D*Z';
    B2h=-1/6*[y2D';y2D';y2D']-9/40/omega*y2D*Z';
    Eh=81/1600/omega*(y2D*y2D'+x2D*x2D');
    
    f1t=(f(it1(1))+f(it1(2))+f(it1(3)))/3; f2t=(f(it2(1))+f(it2(2))+f(it2(3)))/3;
    f1b=9/20*tarea*f1t;                    f2b=9/20*tarea*f2t;
    b1h=tarea/3*[f1t;f1t;f1t]-f1b/omega*Z; b2h=tarea/3*[f2t;f2t;f2t]-f2b/omega*Z;
    ch=-9/40/omega*(f1b*x2D+f2b*y2D);
    ah=tarea/3*e;

    % assembly global data
    A(it1,it1)=A(it1,it1)+Ah;  A(it2,it2)=A(it2,it2)+Ah;
    B(itB,it1)=B(itB,it1)+B1h; B(itB,it2)=B(itB,it2)+B2h;
    E(itB,itB)=E(itB,itB)+Eh;
    b(it1)=b(it1)+b1h; b(it2)=b(it2)+b2h;
    c(itB)=c(itB)+ch; a(itB)=a(itB)+ah;
end