function [A,B,E,b,c,a]=assemblyP1bP1D(p,t,nu,alpha,f)
% Stokes problem in 2D with P1-bubble/P1 elements. 
% For symmetric part of the gradient.
% Loop over triangles.
%
% Input:
% p ... fem nodes
% t ... fem triangles (counterclockwise oriented)
% alpha ... constant at mass matrix 
% nu ... viscosity constant
% f ... force at nodes
%
% Output:
% A ... alpha*(mass matrix)+nu*(diffusion matrix) 
%       for both velocity components u=[u1(1),u2(1),u1(2),u2(2),...]
% B ... divergence matrix
% E ... stabilization matrix
% b,c ... components of right-hand side vector
% a ... integral weights over domain
%
% Structure of the system: | A  B'| |u| = | b |
%                          | B -E | |p|   | c |
% with A = | A11 A12 |
%          | A21 A22 |
% being symmetric

np=size(p,1); nt=size(t,1);
A=sparse(2*np,2*np); B=sparse(np,2*np); E=sparse(np,np);
b=zeros(2*np,1); c=zeros(np,1); a=zeros(np,1);
Mel=1/12*[2 1 1; 1 2 1; 1 1 2]; e=[1;1;1];

for ih=1:nt
    itB=t(ih,1:3); it1=2*itB-1; it2=it1+1; it12=[it1,it2];
    x21=p(t(ih,2),1)-p(t(ih,1),1); y12=p(t(ih,1),2)-p(t(ih,2),2);
    x32=p(t(ih,3),1)-p(t(ih,2),1); y23=p(t(ih,2),2)-p(t(ih,3),2);
    x13=p(t(ih,1),1)-p(t(ih,3),1); y31=p(t(ih,3),2)-p(t(ih,1),2);    
    x2D=[y23; y31; y12]; y2D=[x32; x13; x21]; 
    tarea=(x21*x2D(2)-x13*x2D(3))/2;
    Z=3/20*alpha*tarea*e; Z=[Z,zeros(3,1);zeros(3,1),Z];
    omega11=81*nu/20/tarea*(x2D(1)^2+y2D(1)^2/2-x2D(2)*x2D(3)-y2D(2)*y2D(3)/2)+81*alpha*tarea/280;
    omega22=81*nu/20/tarea*(x2D(1)^2/2+y2D(1)^2-x2D(2)*x2D(3)/2-y2D(2)*y2D(3))+81*alpha*tarea/280;
    omega12=81*nu/80/tarea*y2D'*x2D; omega21=omega12;

    % element matrices
    omega_det=omega11*omega22-omega12*omega21;
    invomega=[omega22/omega_det -omega12/omega_det; -omega12/omega_det omega11/omega_det];
    M=alpha*tarea*Mel;
    Ah=[M+nu/2/tarea*(x2D*x2D'+y2D*y2D'/2) nu/4/tarea*y2D*x2D'
        nu/4/tarea*x2D*y2D'              M+nu/2/tarea*(x2D*x2D'/2+y2D*y2D')];
    Ah=Ah-Z*invomega*Z';
    Bh=-1/6*[[x2D';x2D';x2D'],[y2D';y2D';y2D']];
    Bbh=9/40*[x2D,y2D];
    Bh=Bh-Bbh*invomega*Z';
    Eh=Bbh*invomega*Bbh';
    
    % element vectors
    f1t=(f(it1(1))+f(it1(2))+f(it1(3)))/3; f2t=(f(it2(1))+f(it2(2))+f(it2(3)))/3;
    fh=tarea/3*[f1t;f1t;f1t;f2t;f2t;f2t];  fbh=9/20*tarea*[f1t;f2t];
    bh=fh-Z*invomega*fbh;              
    ch=-Bbh*invomega*fbh;
    ah=tarea/3*e;

    % assembly global data
    A(it12,it12)=A(it12,it12)+Ah;
    B(itB,it12)=B(itB,it12)+Bh;
    E(itB,itB)=E(itB,itB)+Eh;
    b(it12)=b(it12)+bh;
    c(itB)=c(itB)+ch; a(itB)=a(itB)+ah;
end