function [A,B,E,b,c,a]=assemblyP1bP1D_vec(p,t,nu,alpha,f)
% Stokes problem in 2D with P1-bubble/P1 elements. 
% For symmetric part of gradient.
% Vectorized code.
%
% Input:
% p ... fem nodes
% t ... fem triangles (counterclockwise oriented)
% alpha ... constant at the mass matrix 
% nu ... viscosity constant
% f ... force at nodes
%
% Output:
% A ... alpha*(mass matrix)+nu*(diffusion matrix) 
%       for both velocity components u=[u1(1),u2(1),u1(2),u2(2),...]
% B ... divergence matrix
% E ... stabilization matrix
% b,c ... components of right-hand side vector
% a ... integral weights over domain (vector)
%
% Structure of the system: | A  B'| |u| = | b |
%                          | B -E | |p|   | c |
% with A = | A11 A12 |
%          | A21 A22 |
% being symmetric

t_even=2*t; t_odd=t_even-1; tt=[t_odd;t_even];
np=size(p,1); np2=2*np;
A=sparse(np2,np2); B=sparse(np,np2); E=sparse(np,np);
b=zeros(np2,1); c=zeros(np,1); a=zeros(np,1);
x21=p(t(:,2),1)-p(t(:,1),1); y12=p(t(:,1),2)-p(t(:,2),2);
x32=p(t(:,3),1)-p(t(:,2),1); y23=p(t(:,2),2)-p(t(:,3),2);
x13=p(t(:,1),1)-p(t(:,3),1); y31=p(t(:,3),2)-p(t(:,1),2);
x2D=[y23 y31 y12]; y2D=[x32 x13 x21];
tarea=(x21.*x2D(:,2)-x13.*x2D(:,3))/2; alta=alpha*tarea;
    
x2D12=x2D(:,1).^2; y2D12=y2D(:,1).^2; x2D2x2D3=x2D(:,2).*x2D(:,3); y2D2y2D3=y2D(:,2).*y2D(:,3);
omega11=81*nu/20*(x2D12+y2D12/2-x2D2x2D3-y2D2y2D3/2)./tarea+81/280*alta;
omega22=81*nu/20*(x2D12/2+y2D12-x2D2x2D3/2-y2D2y2D3)./tarea+81/280*alta;
omega12=81*nu/80*(x2D(:,1).*y2D(:,1)+x2D(:,2).*y2D(:,2)+x2D(:,3).*y2D(:,3))./tarea;
omega_det=omega11.*omega22-omega12.^2;
invomega11=omega22./omega_det;
invomega22=omega11./omega_det;
invomega12=-omega12./omega_det;

% Mass matrix
aux=alta/12;
for i=1:3
    for j=1:i
        A=A+sparse(tt(:,i),tt(:,j),[aux;aux],np2,np2);
    end
end
A=A+A';

% Diffusion matrix
n4ta=nu/4./tarea;
for i=1:3
    n4taxi=n4ta.*x2D(:,i); n4tayi=n4ta.*y2D(:,i);
    for j=i:3 % A11, A22 
        n4taxixj=n4taxi.*x2D(:,j); n4tayiyj=n4tayi.*y2D(:,j);
        aux1=2*n4taxixj+n4tayiyj; aux2=n4taxixj+2*n4tayiyj;
        if i==j
            A=A+sparse(tt(:,i),tt(:,j),[aux1;aux2],np2,np2);
        else
            A=A+sparse(tt(:,i),tt(:,j),[aux1;aux2],np2,np2);
            A=A+sparse(tt(:,j),tt(:,i),[aux1;aux2],np2,np2);
        end
    end
    for j=1:3 % A12, A21
        aux=n4tayi.*x2D(:,j);
        A=A+sparse(t_odd(:,i),t_even(:,j),aux,np2,np2);
        A=A+sparse(t_even(:,j),t_odd(:,i),aux,np2,np2);
    end
end

% Divergence matrix
for j=1:3
    aux=[x2D(:,j)/6;y2D(:,j)/6];
    for i=1:3 
        B=B-sparse([t(:,i);t(:,i)],tt(:,j),aux,np,np2);
    end
end

% Bubble components elimination in A
Z=3/20*alta; aux=Z.*Z; 
aux11=aux.*invomega11; aux22=aux.*invomega22; aux12=aux.*invomega12;
for i=1:3
    for j=1:3
        A=A-sparse(tt(:,i),tt(:,j),[aux11;aux22],np2,np2);
        A=A-sparse(t_odd(:,i),t_even(:,j),aux12,np2,np2);
        A=A-sparse(t_even(:,i),t_odd(:,j),aux12,np2,np2);
    end
end

% Bubble components elimination in B, E 
aux11=9/40.*invomega11; aux22=9/40.*invomega22; aux12=9/40.*invomega12;
for i=1:3
    aux1=aux11.*x2D(:,i)+aux12.*y2D(:,i); 
    aux1z=aux1.*Z;
    aux2=aux12.*x2D(:,i)+aux22.*y2D(:,i); 
    aux2z=aux2.*Z;
    for j=1:3
        E=E+sparse(t(:,i),t(:,j),9/40*(aux1.*x2D(:,j)+aux2.*y2D(:,j)),np,np);
        B=B-sparse([t(:,i);t(:,i)],tt(:,j),[aux1z;aux2z],np,np2);
    end
end

% Right hand-side vector b
f1=f(1:2:np2); f2=f(2:2:np2);
f1=tarea.*(f1(t(:,1))+f1(t(:,2))+f1(t(:,3)))/9 ; 
f2=tarea.*(f2(t(:,1))+f2(t(:,2))+f2(t(:,3)))/9;
for i=1:3
    b=b+sparse(tt(:,i),1,[f1;f2],np2,1);
end

% Bubble components elimination in b, c and a
f1=27/20*f1;                       f2=27/20*f2; 
f1b=invomega11.*f1+invomega12.*f2; f2b=invomega12.*f1+invomega22.*f2;
f1bz=f1b.*Z;                       f2bz=f2b.*Z;

for i=1:3
    b=b-sparse(tt(:,i),1,[f1bz;f2bz],np2,1);
    c=c-sparse(t(:,i),1,9/40*(f1b.*x2D(:,i)+f2b.*y2D(:,i)),np,1);
    a=a+sparse(t(:,i),1,tarea/3,np,1);
end