function [A,B,E,b,c,a]=assemblyP1bP1D_vecO(p,t,nu,alpha,f)
% Stokes problem with P1-bubble/P1 elements.
% For symmetric part of gradient.
% Vectorized code.
% Optimized version.
%
% Input:
% p ... fem nodes
% t ... fem triangles (counterclockwise oriented)
% alpha ... constant at the mass matrix
% nu ... viscosity constant
% f ... force at nodes
%
% Output:
% A ... alpha*(mass matrix)+nu*(diffusion matrix)
%       for both velocity components u=[u1(1),u2(1),u1(2),u2(2),...]
% B ... divergence matrix
% E ... stabilization matrix
% b,c ... components of right-hand side vector
% a ... integral weights over domain (vector)
%
% Structure of the system: | A  B'| |u| = | b |
%                          | B -E | |p|   | c |
% with A = | A11 A12 |
%          | A21 A22 |
% being symmetric
np=size(p,1);
% 0. triangles area
x21=p(t(:,2),1)-p(t(:,1),1); y12=p(t(:,1),2)-p(t(:,2),2);
x32=p(t(:,3),1)-p(t(:,2),1); y23=p(t(:,2),2)-p(t(:,3),2);
x13=p(t(:,1),1)-p(t(:,3),1); y31=p(t(:,3),2)-p(t(:,1),2);
tarea=(x21.*y31-x13.*y12)/2;
xt=[y23 y31 y12]; yt=[x32 x13 x21];

nut=nu./tarea;
mt=(3/20)*alpha*tarea;

omega_12=(81/80)*nut.*(y23.*x32+y31.*x13+y12.*x21);                             
omega_11=((81*alpha)/280)*tarea+(81/20)*nut.*(y23.^2-y31.*y12+0.5*(x32.^2-x13.*x21));                         
omega_22=((81*alpha)/280)*tarea+(81/20)*nut.*((1/2)*(y23.^2-y31.*y12)+x32.^2-x13.*x21);                      
invomega_N=1./(omega_11.*omega_22-omega_12.^2);              

am_12=omega_12.*invomega_N;
am_11=omega_11.*invomega_N;
am_22=omega_22.*invomega_N;

f1=f(1:2:end-1); f2=f(2:2:end);
f1t=tarea.*(f1(t(:,1))+f1(t(:,2))+f1(t(:,3)))/9; 
f2t=tarea.*(f2(t(:,1))+f2(t(:,2))+f2(t(:,3)))/9; 

f1=(27/20)*(am_22.*f1t-am_12.*f2t);
f2=(27/20)*(am_11.*f2t-am_12.*f1t);

f1t=f1t-mt.*f1;
f2t=f2t-mt.*f2;

A1=sparse(np,np); A2=sparse(np,np);
A12=sparse(np,np);E=sparse(np,np);
B1=sparse(np,np); B2=sparse(np,np);
b1=zeros(np,1); b2=zeros(np,1); 
c=zeros(np,1);a=zeros(np,1);
for i=1:3
    a=a+sparse(t(:,i),1,tarea/3,np,1);
    % the right side vector components
    b1=b1+sparse(t(:,i),1,f1t,np,1);
    b2=b2+sparse(t(:,i),1,f2t,np,1);
    c=c-sparse(t(:,i),1,(9/40)*(xt(:,i).*f1+yt(:,i).*f2),np,1);
    % parts for matrices indemended on j
    E1=(9/40)*(am_22.*xt(:,i)-am_12.*yt(:,i));
    E2=(9/40)*(am_11.*yt(:,i)-am_12.*xt(:,i));
    for j=1:3
        if(i==j)  
        A1=A1+sparse(t(:,i),t(:,j),(alpha/6)*tarea+(nut/4).*(yt(:,i).*yt(:,j)+2*xt(:,i).*xt(:,j))-(mt.^2).*am_22,np,np); 
        A2=A2+sparse(t(:,i),t(:,j),(alpha/6)*tarea+(nut/4).*(2*yt(:,i).*yt(:,j)+xt(:,i).*xt(:,j))-(mt.^2).*am_11,np,np);
        else
        A1=A1+sparse(t(:,i),t(:,j),(alpha/12)*tarea+(nut/4).*(yt(:,i).*yt(:,j)+2*xt(:,i).*xt(:,j))-(mt.^2).*am_22,np,np); 
        A2=A2+sparse(t(:,i),t(:,j),(alpha/12)*tarea+(nut/4).*(2*yt(:,i).*yt(:,j)+xt(:,i).*xt(:,j))-(mt.^2).*am_11,np,np);
        end
        A12=A12+sparse(t(:,i),t(:,j),(nut/4).*yt(:,i).*xt(:,j)+(mt.^2).*am_12,np,np);
        B1=B1-sparse(t(:,i),t(:,j),xt(:,j)/6+mt.*E1,np,np);
        B2=B2-sparse(t(:,i),t(:,j),yt(:,j)/6+mt.*E2,np,np);          
        E=E+sparse(t(:,i),t(:,j),(9/40)*(E1.*xt(:,j)+E2.*yt(:,j)),np,np);
    end
end
t2(1:2:2*np-1)=1:np; t2(2:2:2*np)=np+(1:np);
A=[A1 A12; A12' A2]; A=A(t2,t2);
B=[B1 B2]; B=B(:,t2);
b=[b1;b2]; b=b(t2,1);
end