function [A,B,E,b,c,a]=assemblyP1bP1_vec(p,t,nu,alpha,f)
% Stokes problem in 2D with P1-bubble/P1 elements.
% For Laplace operator.
% Vectorized code.
%
% Input:
% p ... fem nodes
% t ... fem triangles (counterclockwise oriented)
% alpha ... constant at mass matrix 
% nu ... viscosity constant
% f ... force at nodes
%
% Output:
% A ... alpha*(mass matrix)+nu*(diffusion matrix) 
%       for both velocity components u=[u1(1),u2(1),u1(2),u2(2),...]
% B ... divergence matrix
% E ... stabilization matrix
% b,c ... components of right-hand side vector
% a ... integral weights over domain
%
% Structure of the system: | A  B'| |u| = | b |
%                          | B -E | |p|   | c |
% with A = | A11 0   |
%          | 0   A22 |
% A11=A22 being symmetric

tt=2*t; tt=[tt-1;tt];
np=size(p,1); np2=2*np;
A=sparse(np2,np2); B=sparse(np,np2); E=sparse(np,np);
b=zeros(np2,1); c=zeros(np,1); a=zeros(np,1);
x21=p(t(:,2),1)-p(t(:,1),1); y12=p(t(:,1),2)-p(t(:,2),2);
x32=p(t(:,3),1)-p(t(:,2),1); y23=p(t(:,2),2)-p(t(:,3),2);
x13=p(t(:,1),1)-p(t(:,3),1); y31=p(t(:,3),2)-p(t(:,1),2);
x2D=[y23 y31 y12]; y2D=[x32 x13 x21];
tarea=(x21.*x2D(:,2)-x13.*x2D(:,3))/2; alta=alpha*tarea;
omega=81/40*nu*(x2D(:,1).^2+y2D(:,1).^2-x2D(:,2).*x2D(:,3)-y2D(:,2).*y2D(:,3))./tarea+81/280*alta;

% Mass matrix
aux=alta/12;
for i=1:3
    for j=1:i
        A=A+sparse(tt(:,i),tt(:,j),[aux;aux],np2,np2);
    end
end
A=A+A';

% Diffusion matrix
n4ta=nu/4./tarea;
for i=1:3
    n4taxi=n4ta.*x2D(:,i); n4tayi=n4ta.*y2D(:,i);
    for j=i:3 % A11, A22=A11 
        aux=n4taxi.*x2D(:,j)+n4tayi.*y2D(:,j);
        if i==j
            A=A+sparse(tt(:,i),tt(:,j),[aux;aux],np2,np2);
        else
            A=A+sparse(tt(:,i),tt(:,j),[aux;aux],np2,np2);
            A=A+sparse(tt(:,j),tt(:,i),[aux;aux],np2,np2);
        end
    end
end

% Divergence matrix
for j=1:3
    aux=[x2D(:,j)/6;y2D(:,j)/6];
    for i=1:3 
        B=B-sparse([t(:,i);t(:,i)],tt(:,j),aux,np,np2);
    end
end

% Bubble components elimination in A
Z=3/20*alta; aux=Z.*Z./omega;
for i=1:3
    for j=1:3
        A=A-sparse(tt(:,i),tt(:,j),[aux;aux],np2,np2);
    end
end

% Bubble components elimination in B, E 
aux=9/40./omega;
for i=1:3
    aux1=aux.*y2D(:,i); aux1z=aux1.*Z;
    aux2=aux.*x2D(:,i); aux2z=aux2.*Z;
    for j=1:3
        E=E+sparse(t(:,i),t(:,j),9/40*(aux1.*y2D(:,j)+aux2.*x2D(:,j)),np,np);
        B=B-sparse([t(:,i);t(:,i)],tt(:,j),[aux2z;aux1z],np,np2);
    end
end

% Right hand-side vector b
f1=f(1:2:np2); f2=f(2:2:np2);
f1=tarea.*(f1(t(:,1))+f1(t(:,2))+f1(t(:,3)))/9; 
f2=tarea.*(f2(t(:,1))+f2(t(:,2))+f2(t(:,3)))/9;
for i=1:3
    b=b+sparse(tt(:,i),1,[f1;f2],np2,1);
end

% Bubble components elimination in b, c and a
aux=27/20./omega; f1b=aux.*f1; f2b=aux.*f2; 
for i=1:3
    b=b-sparse(tt(:,i),1,[Z.*f1b;Z.*f2b],np2,1);
    c=c-sparse(t(:,i),1,9/40*(f1b.*x2D(:,i)+f2b.*y2D(:,i)),np,1);
    a=a+sparse(t(:,i),1,tarea/3,np,1);
end