function [A,Bu,Bl,E,b,c,a]=assembly3DP1bP1CN_vecO(p,t,nu,alpha,f,w)
% Navier-Stokes problem in 3D with P1-bubble/P1 elements for Newton linearization.
% For Laplace operator.
% Vectorized code.
% Optimized version.
%
% Input:
% p ... fem nodes
% t ... fem tetrahedras
% alpha ... constant at mass matrix
% nu ... viscosity constant
% f ... force at nodes
% w ... given velocity vector
%
% Output:
% A ... alpha*(mass matrix)+nu*(diffusion matrix)+convection matrix(Newton)
%       for three velocity components u=[u1(1),u2(1),u3(1),u1(2),u2(2),u3(2),...]
% Bu, Bl ... divergence/gradient matrices
% E ... stabilization matrix
% b,c ... components of right-hand side vector
% a ... integral weights over domain
%
% Structure of the system: | A  Bu'| |u| = | b |
%                          | Bl -E | |p|   | c |
% with A = | A11 A12 A13 |
%          | A21 A22 A23 |
%          | A31 A32 A33 |

np=size(p,1);

x21=p(t(:,2),1)-p(t(:,1),1); y21=p(t(:,2),2)-p(t(:,1),2); z21=p(t(:,2),3)-p(t(:,1),3);
x31=p(t(:,3),1)-p(t(:,1),1); y31=p(t(:,3),2)-p(t(:,1),2); z31=p(t(:,3),3)-p(t(:,1),3);
x41=p(t(:,4),1)-p(t(:,1),1); y41=p(t(:,4),2)-p(t(:,1),2); z41=p(t(:,4),3)-p(t(:,1),3);
x32=p(t(:,3),1)-p(t(:,2),1); y32=p(t(:,3),2)-p(t(:,2),2); z32=p(t(:,3),3)-p(t(:,2),3);
x42=p(t(:,4),1)-p(t(:,2),1); y42=p(t(:,4),2)-p(t(:,2),2); z42=p(t(:,4),3)-p(t(:,2),3);
xt=[z32.*y42-y32.*z42,y31.*z41-z31.*y41,z21.*y41-y21.*z41,y21.*z31-z21.*y31];
yt=[x32.*z42-z32.*x42,z31.*x41-x31.*z41,x21.*z41-z21.*x41,z21.*x31-x21.*z31];
zt=[y32.*x42-x32.*y42,x31.*y41-y31.*x41,y21.*x41-x21.*y41,x21.*y31-y21.*x31];

xtt=xt(:,2).*xt(:,3)+xt(:,2).*xt(:,4)+xt(:,3).*xt(:,4)+yt(:,2).*yt(:,3)+yt(:,2).*yt(:,4)+yt(:,3).*yt(:,4)+zt(:,2).*zt(:,3)+zt(:,2).*zt(:,4)+zt(:,3).*zt(:,4);
tvolume=(x21.*xt(:,2)+x31.*xt(:,3)+x41.*xt(:,4))/6;
msgn=sign(tvolume);
tvolume=abs(tvolume);

w1=w(1:3:end-2); w2=w(2:3:end-1); w3=w(3:3:end);
w1t=(w1(t(:,1))+w1(t(:,2))+w1(t(:,3))+w1(t(:,4)))/4;
w2t=(w2(t(:,1))+w2(t(:,2))+w2(t(:,3))+w2(t(:,4)))/4;
w3t=(w3(t(:,1))+w3(t(:,2))+w3(t(:,3))+w3(t(:,4)))/4;

cal=(1/24)*msgn.*[w1t.*xt(:,1)+w2t.*yt(:,1)+w3t.*zt(:,1),...
    w1t.*xt(:,2)+w2t.*yt(:,2)+w3t.*zt(:,2),...
    w1t.*xt(:,3)+w2t.*yt(:,3)+w3t.*zt(:,3),...
    w1t.*xt(:,4)+w2t.*yt(:,4)+w3t.*zt(:,4)];

w211 = w1(t(:,2))-w1(t(:,1));
w311 = w1(t(:,3))-w1(t(:,1));
w411 = w1(t(:,4))-w1(t(:,1));
w212 = w2(t(:,2))-w2(t(:,1));
w312 = w2(t(:,3))-w2(t(:,1));
w412 = w2(t(:,4))-w2(t(:,1));
w213 = w3(t(:,2))-w3(t(:,1));
w313 = w3(t(:,3))-w3(t(:,1));
w413 = w3(t(:,4))-w3(t(:,1));

w1x = (msgn/6).*(w211.*(y31.*z41-z31.*y41)+w311.*(z21.*y41-y21.*z41)+w411.*(y21.*z31-z21.*y31));
w2x = (msgn/6).*(w212.*(y31.*z41-z31.*y41)+w312.*(z21.*y41-y21.*z41)+w412.*(y21.*z31-z21.*y31));
w3x = (msgn/6).*(w213.*(y31.*z41-z31.*y41)+w313.*(z21.*y41-y21.*z41)+w413.*(y21.*z31-z21.*y31));
w1y = (msgn/6).*(x21.*(w311.*z41-z31.*w411)+x31.*(z21.*w411-w211.*z41)+x41.*(w211.*z31-z21.*w311));
w2y = (msgn/6).*(x21.*(w312.*z41-z31.*w412)+x31.*(z21.*w412-w212.*z41)+x41.*(w212.*z31-z21.*w312));
w3y = (msgn/6).*(x21.*(w313.*z41-z31.*w413)+x31.*(z21.*w413-w213.*z41)+x41.*(w213.*z31-z21.*w313));
w1z = (msgn/6).*(x21.*(y31.*w411-w311.*y41)+x31.*(w211.*y41-y21.*w411)+x41.*(y21.*w311-w211.*y31));
w2z = (msgn/6).*(x21.*(y31.*w412-w312.*y41)+x31.*(w212.*y41-y21.*w412)+x41.*(y21.*w312-w212.*y31));
w3z = (msgn/6).*(x21.*(y31.*w413-w313.*y41)+x31.*(w213.*y41-y21.*w413)+x41.*(y21.*w313-w213.*y31));

nut=nu./tvolume;
omega_MR=((8192*alpha)/51975)*tvolume+(2048/8505)*nut.*(xt(:,1).^2+yt(:,1).^2+zt(:,1).^2-xtt);

omega_11 = omega_MR + (8192/51975)*w1x;
omega_12 =         (8192/51975)*w1y;
omega_13 =         (8192/51975)*w1z;

omega_21 =         (8192/51975)*w2x;
omega_22 = omega_MR + (8192/51975)*w2y;
omega_23 =         (8192/51975)*w2z;

omega_31 =         (8192/51975)*w3x;
omega_32 =         (8192/51975)*w3y;
omega_33 = omega_MR + (8192/51975)*w3z;


invomega_N=1./(omega_11.*(omega_22.*omega_33-omega_23.*omega_32)...
    +omega_12.*(omega_23.*omega_31-omega_21.*omega_33)...
    +omega_13.*(omega_21.*omega_32-omega_22.*omega_31));

am_11=(omega_22.*omega_33-omega_23.*omega_32).*invomega_N;
am_12=(omega_13.*omega_32-omega_12.*omega_33).*invomega_N;
am_13=(omega_12.*omega_23-omega_13.*omega_22).*invomega_N;

am_21=(omega_23.*omega_31-omega_21.*omega_33).*invomega_N;
am_22=(omega_11.*omega_33-omega_13.*omega_31).*invomega_N;
am_23=(omega_13.*omega_21-omega_11.*omega_23).*invomega_N;

am_31=(omega_21.*omega_32-omega_22.*omega_31).*invomega_N;
am_32=(omega_12.*omega_31-omega_11.*omega_32).*invomega_N;
am_33=(omega_11.*omega_22-omega_12.*omega_21).*invomega_N;

zu=((8*alpha)/105)*tvolume-(16/315)*msgn.*(w1t.*xt+w2t.*yt+w3t.*zt);
zl = ((16*alpha)/105)*tvolume-zu;

zu_11 = zu + (8/105).*w1x;
zu_22 = zu + (8/105).*w2y;
zu_33 = zu + (8/105).*w3z;

zl_11 = zl + (8/105).*w1x;
zl_22 = zl + (8/105).*w2y;
zl_33 = zl + (8/105).*w3z;

z_12 = (8/105).*w1y;
z_13 = (8/105).*w1z;
z_21 = (8/105).*w2x;
z_23 = (8/105).*w2z;
z_31 = (8/105).*w3x;
z_32 = (8/105).*w3y;

f1=f(1:3:end-2); f2=f(2:3:end-1); f3=f(3:3:end);
f1t=(1/4)*(tvolume.*(f1(t(:,1))+f1(t(:,2))+f1(t(:,3))+f1(t(:,4)))/4 + w1x.*w1t + w1y.*w2t + w1z.*w3t);
f2t=(1/4)*(tvolume.*(f2(t(:,1))+f2(t(:,2))+f2(t(:,3))+f2(t(:,4)))/4 + w2x.*w1t + w2y.*w2t + w2z.*w3t);
f3t=(1/4)*(tvolume.*(f3(t(:,1))+f3(t(:,2))+f3(t(:,3))+f3(t(:,4)))/4 + w3x.*w1t + w3y.*w2t + w3z.*w3t);

f1b = (128/105)*(am_11.*f1t+am_12.*f2t+am_13.*f3t);
f2b = (128/105)*(am_21.*f1t+am_22.*f2t+am_23.*f3t);
f3b = (128/105)*(am_31.*f1t+am_32.*f2t+am_33.*f3t);

w1x=(alpha/20)*tvolume+(1/20)*w1x;
w1y=(1/20)*w1y;
w1z=(1/20)*w1z;
w2x=(1/20)*w2x;
w2y=(alpha/20)*tvolume+(1/20)*w2y;
w2z=(1/20)*w2z;
w3x=(1/20)*w3x;
w3y=(1/20)*w3y;
w3z=(alpha/20)*tvolume+(1/20)*w3z;

Ah = sparse(np,np); E=sparse(np,np);
A11=sparse(np,np); A12=sparse(np,np); A13=sparse(np,np);
A21=sparse(np,np); A22=sparse(np,np); A23=sparse(np,np);
A31=sparse(np,np); A32=sparse(np,np); A33=sparse(np,np);
Bu1=sparse(np,np); Bu2=sparse(np,np); Bu3=sparse(np,np);
Bl1=sparse(np,np); Bl2=sparse(np,np); Bl3=sparse(np,np);
b1=zeros(np,1); b2=zeros(np,1); b3=zeros(np,1);
c=zeros(np,1); a=zeros(np,1);
for i=1:4
    a=a+sparse(t(:,i),1,tvolume/4,np,1);
    b1=b1+sparse(t(:,i),1,f1t-zu_11(:,i).*f1b-z_12.*f2b-z_13.*f3b,np,1);
    b2=b2+sparse(t(:,i),1,f2t-z_21.*f1b-zu_22(:,i).*f2b-z_23.*f3b,np,1);
    b3=b3+sparse(t(:,i),1,f3t-z_31.*f1b-z_32.*f2b-zu_33(:,i).*f3b,np,1);
    c=c-sparse(t(:,i),1,(16/315).*msgn.*(xt(:,i).*f1b+yt(:,i).*f2b+zt(:,i).*f3b),np,1);

    l11=(am_11.*zu_11(:,i)+am_21.*z_12+am_31.*z_13);
    l12=(am_12.*zu_11(:,i)+am_22.*z_12+am_32.*z_13);
    l13=(am_13.*zu_11(:,i)+am_23.*z_12+am_33.*z_13);

    l21=(am_11.*z_21+am_21.*zu_22(:,i)+am_31.*z_23);
    l22=(am_12.*z_21+am_22.*zu_22(:,i)+am_32.*z_23);
    l23=(am_13.*z_21+am_23.*zu_22(:,i)+am_33.*z_23);

    l31 = (am_11.*z_31+am_21.*z_32+am_31.*zu_33(:,i));
    l32 = (am_12.*z_31+am_22.*z_32+am_32.*zu_33(:,i));
    l33 = (am_13.*z_31+am_23.*z_32+am_33.*zu_33(:,i));

    %Bu
    B1=(16/315)*(am_11.*xt(:,i)+am_12.*yt(:,i)+am_13.*zt(:,i));
    B2=(16/315)*(am_21.*xt(:,i)+am_22.*yt(:,i)+am_23.*zt(:,i));
    B3=(16/315)*(am_31.*xt(:,i)+am_32.*yt(:,i)+am_33.*zt(:,i));

    E1=(16/315)*(am_11.*xt(:,i)+am_21.*yt(:,i)+am_31.*zt(:,i));
    E2=(16/315)*(am_12.*xt(:,i)+am_22.*yt(:,i)+am_32.*zt(:,i));
    E3=(16/315)*(am_13.*xt(:,i)+am_23.*yt(:,i)+am_33.*zt(:,i));
    for j=1:4
        Ah=Ah+sparse(t(:,i),t(:,j),(nut/36).*(xt(:,i).*xt(:,j)+yt(:,i).*yt(:,j)+zt(:,i).*zt(:,j))+cal(:,j),np,np);
        if(i==j)
            A11=A11+sparse(t(:,i),t(:,j),2*w1x-(zl_11(:,j).*l11+z_12.*l12+z_13.*l13),np,np);
            A12=A12+sparse(t(:,i),t(:,j),2*w1y-(z_21.*l11+zl_22(:,j).*l12+z_23.*l13),np,np);
            A13=A13+sparse(t(:,i),t(:,j),2*w1z-(z_31.*l11+z_32.*l12+zl_33(:,j).*l13),np,np);

            A21=A21+sparse(t(:,i),t(:,j),2*w2x-(zl_11(:,j).*l21+z_12.*l22+z_13.*l23),np,np);
            A22=A22+sparse(t(:,i),t(:,j),2*w2y-(z_21.*l21+zl_22(:,j).*l22+z_23.*l23),np,np);
            A23=A23+sparse(t(:,i),t(:,j),2*w2z-(z_31.*l21+z_32.*l22+zl_33(:,j).*l23),np,np);

            A31=A31+sparse(t(:,i),t(:,j),2*w3x-(zl_11(:,j).*l31+z_12.*l32+z_13.*l33),np,np);
            A32=A32+sparse(t(:,i),t(:,j),2*w3y-(z_21.*l31+zl_22(:,j).*l32+z_23.*l33),np,np);
            A33=A33+sparse(t(:,i),t(:,j),2*w3z-(z_31.*l31+z_32.*l32+zl_33(:,j).*l33),np,np);
        else
            A11=A11+sparse(t(:,i),t(:,j),w1x-(zl_11(:,j).*l11+z_12.*l12+z_13.*l13),np,np);
            A12=A12+sparse(t(:,i),t(:,j),w1y-(z_21.*l11+zl_22(:,j).*l12+z_23.*l13),np,np);
            A13=A13+sparse(t(:,i),t(:,j),w1z-(z_31.*l11+z_32.*l12+zl_33(:,j).*l13),np,np);

            A21=A21+sparse(t(:,i),t(:,j),w2x-(zl_11(:,j).*l21+z_12.*l22+z_13.*l23),np,np);
            A22=A22+sparse(t(:,i),t(:,j),w2y-(z_21.*l21+zl_22(:,j).*l22+z_23.*l23),np,np);
            A23=A23+sparse(t(:,i),t(:,j),w2z-(z_31.*l21+z_32.*l22+zl_33(:,j).*l23),np,np);

            A31=A31+sparse(t(:,i),t(:,j),w3x-(zl_11(:,j).*l31+z_12.*l32+z_13.*l33),np,np);
            A32=A32+sparse(t(:,i),t(:,j),w3y-(z_21.*l31+zl_22(:,j).*l32+z_23.*l33),np,np);
            A33=A33+sparse(t(:,i),t(:,j),w3z-(z_31.*l31+z_32.*l32+zl_33(:,j).*l33),np,np);
        end
        Bu1=Bu1-sparse(t(:,i),t(:,j),msgn.*((1/24)*xt(:,j)+(zu_11(:,j).*B1+z_12.*B2+z_13.*B3)),np,np);
        Bu2=Bu2-sparse(t(:,i),t(:,j),msgn.*((1/24)*yt(:,j)+(z_21.*B1+zu_22(:,j).*B2+z_23.*B3)),np,np);
        Bu3=Bu3-sparse(t(:,i),t(:,j),msgn.*((1/24)*zt(:,j)+(z_31.*B1+z_32.*B2+zu_33(:,j).*B3)),np,np);

        Bl1=Bl1-sparse(t(:,i),t(:,j),msgn.*((1/24)*xt(:,j)+(zl_11(:,j).*E1+z_12.*E2+z_13.*E3)),np,np);
        Bl2=Bl2-sparse(t(:,i),t(:,j),msgn.*((1/24)*yt(:,j)+(z_21.*E1+zl_22(:,j).*E2+z_23.*E3)),np,np);
        Bl3=Bl3-sparse(t(:,i),t(:,j),msgn.*((1/24)*zt(:,j)+(z_31.*E1+z_32.*E2+zl_33(:,j).*E3)),np,np);

        E=E+sparse(t(:,i),t(:,j),(16/315)*(xt(:,j).*E1+yt(:,j).*E2+zt(:,j).*E3),np,np);
    end
end
t3(1:3:3*np-2)=1:np; t3(2:3:3*np-1)=np+(1:np); t3(3:3:3*np)=2*np+(1:np);
A=[Ah+A11 A12 A13;
   A21 Ah+A22 A23;
   A31 A32 Ah+A33]; A=A(t3,t3);
Bu=[Bu1 Bu2 Bu3]; Bu=Bu(:,t3);
Bl=[Bl1 Bl2 Bl3]; Bl=Bl(:,t3);
b=[b1;b2;b3]; b=b(t3,1);
end