function [A,Bu,Bl,E,b,c,a]=assemblyP1bP1CN_3(p,t,nu,alpha,f,w)
% Navier-Stokes problem in 3D with P1-bubble/P1 elements for Newton linearization.
% For Laplace operator.
% Loop over tetrahedras.
% 
% Input:
% p ... fem nodes
% t ... fem tetrahedras
% alpha ... constant at mass matrix 
% nu ... viscosity constant
% f ... force at nodes
% w ... given velocity vector
%
% Output:
% A ... alpha*(mass matrix)+nu*(diffusion matrix)+convective matrix(Newton) 
%       for three velocity components u=[u1(1),u2(1),u3(1),u1(2),u2(2),u3(2),...]
% Bu, Bl ... divergence/gradient matrices
% E ... stabilization matrix
% b,c ... components of right-hand side vector
% a ... integral weights over domain
%
% Structure of the system: | A  Bu'| |u| = | b |
%                          | Bl -E | |p|   | c |
% with A = | A11 A12 A13 |
%          | A21 A22 A23 |
%          | A31 A32 A33 |

np=size(p,1); nt=size(t,1);
A=sparse(3*np,3*np); Bu=sparse(np,3*np); Bl=sparse(np,3*np); E=sparse(np,np);
b=zeros(3*np,1); c=zeros(np,1); a=zeros(np,1);
Mel=1/20*[2 1 1 1; 1 2 1 1; 1 1 2 1; 1 1 1 2]; e=[1;1;1;1];
w1=w(1:3:end-2); w2=w(2:3:end-1); w3=w(3:3:end);

for ih=1:nt
    itB=t(ih,1:4); it1=3*itB-2; it2=it1+1; it3=it1+2; it123=[it1,it2,it3];
    x21=p(itB(2),1)-p(itB(1),1); y21=p(itB(2),2)-p(itB(1),2); z21=p(itB(2),3)-p(itB(1),3);
    x31=p(itB(3),1)-p(itB(1),1); y31=p(itB(3),2)-p(itB(1),2); z31=p(itB(3),3)-p(itB(1),3);
    x41=p(itB(4),1)-p(itB(1),1); y41=p(itB(4),2)-p(itB(1),2); z41=p(itB(4),3)-p(itB(1),3);
    x32=p(itB(3),1)-p(itB(2),1); y32=p(itB(3),2)-p(itB(2),2); z32=p(itB(3),3)-p(itB(2),3);
    x42=p(itB(4),1)-p(itB(2),1); y42=p(itB(4),2)-p(itB(2),2); z42=p(itB(4),3)-p(itB(2),3);
    x3D=[y42*z32-y32*z42; y31*z41-y41*z31; y41*z21-y21*z41; y21*z31-y31*z21];    
    y3D=[x32*z42-x42*z32; x41*z31-x31*z41; x21*z41-x41*z21; x31*z21-x21*z31];    
    z3D=[x42*y32-x32*y42; x31*y41-x41*y31; x41*y21-x21*y41; x21*y31-x31*y21];
    tvolume=(x21*x3D(2)+x31*x3D(3)+x41*x3D(4))/6; detX=6*tvolume;
    msgn=sign(tvolume); tvolume=abs(tvolume);
 
    % for convective matrix (Newton)
    w11=w1(t(ih,1)); w12=w1(t(ih,2)); w13=w1(t(ih,3)); w14=w1(t(ih,4));
    w21=w2(t(ih,1)); w22=w2(t(ih,2)); w23=w2(t(ih,3)); w24=w2(t(ih,4));
    w31=w3(t(ih,1)); w32=w3(t(ih,2)); w33=w3(t(ih,3)); w34=w3(t(ih,4));
    w1t=(w11+w12+w13+w14)/4; w2t=(w21+w22+w23+w24)/4;  w3t=(w31+w32+w33+w34)/4;
    dxw1=((w12-w11)*x3D(2)+(w13-w11)*x3D(3)+(w14-w11)*x3D(4))/detX;
    dyw1=((w12-w11)*y3D(2)+(w13-w11)*y3D(3)+(w14-w11)*y3D(4))/detX;
    dzw1=((w12-w11)*z3D(2)+(w13-w11)*z3D(3)+(w14-w11)*z3D(4))/detX;
    dxw2=((w22-w21)*x3D(2)+(w23-w21)*x3D(3)+(w24-w21)*x3D(4))/detX;
    dyw2=((w22-w21)*y3D(2)+(w23-w21)*y3D(3)+(w24-w21)*y3D(4))/detX;
    dzw2=((w22-w21)*z3D(2)+(w23-w21)*z3D(3)+(w24-w21)*z3D(4))/detX;
    dxw3=((w32-w31)*x3D(2)+(w33-w31)*x3D(3)+(w34-w31)*x3D(4))/detX;
    dyw3=((w32-w31)*y3D(2)+(w33-w31)*y3D(3)+(w34-w31)*y3D(4))/detX;
    dzw3=((w32-w31)*z3D(2)+(w33-w31)*z3D(3)+(w34-w31)*z3D(4))/detX;

    mt1=8/105*tvolume*e; mt=alpha*mt1;
    cl=16/315*msgn*(w1t*x3D+w2t*y3D+w3t*z3D);
    Zu=mt-cl; Zu=[Zu+dxw1*mt1    dyw1*mt1     dzw1*mt1; 
                     dxw2*mt1 Zu+dyw2*mt1     dzw2*mt1
                     dxw3*mt1    dyw3*mt1  Zu+dzw3*mt1];
    Zl=mt+cl; Zl=[Zl+dxw1*mt1    dyw1*mt1     dzw1*mt1; 
                     dxw2*mt1 Zl+dyw2*mt1     dzw2*mt1
                     dxw3*mt1    dyw3*mt1  Zl+dzw3*mt1];
        
    omegaM1=8192*tvolume/51975;
    xyz3D=x3D(2)*(x3D(3)+x3D(4))+x3D(3)*x3D(4)+y3D(2)*(y3D(3)+y3D(4))+y3D(3)*y3D(4)+z3D(2)*(z3D(3)+z3D(4))+z3D(3)*z3D(4);
    omega=2048/8505/tvolume*nu*(x3D(1)^2+y3D(1)^2+z3D(1)^2-xyz3D)+alpha*omegaM1;
    
    omega11=omega+dxw1*omegaM1; omega12=      dyw1*omegaM1; omega13=      dzw1*omegaM1;
    omega21=      dxw2*omegaM1; omega22=omega+dyw2*omegaM1; omega23=      dzw2*omegaM1;
    omega31=      dxw3*omegaM1; omega32=      dyw3*omegaM1; omega33=omega+dzw3*omegaM1;
    
    invomega=[omega22*omega33-omega23*omega32, -omega12*omega33+omega32*omega13, omega12*omega23-omega22*omega13
              -omega21*omega33+omega31*omega23, omega11*omega33-omega13*omega31, -omega11*omega23+omega21*omega13
              omega21*omega32-omega31*omega22, -omega11*omega32+omega31*omega12, omega11*omega22-omega12*omega21];
    omega_det=omega11*invomega(1,1)+omega12*invomega(2,1)+omega13*invomega(3,1);
    invomega=invomega/omega_det;
    
    % element matrices and vectors
    R=nu/36/tvolume*(x3D*x3D'+y3D*y3D'+z3D*z3D');
    M1=tvolume*Mel;
    C=msgn/24*(w1t*[x3D';x3D';x3D';x3D']+w2t*[y3D';y3D';y3D';y3D']+w3t*[z3D';z3D';z3D';z3D']);
    Ah=[R+C+(alpha+dxw1)*M1            dyw1*M1             dzw1*M1
                   dxw2*M1  R+C+(alpha+dyw2)*M1            dzw2*M1
                   dxw3*M1             dyw3*M1  R+C+(alpha+dzw3)*M1];
    Ah=Ah-Zu*invomega*Zl';
    Bh=-msgn/24*[[x3D';x3D';x3D';x3D'],[y3D';y3D';y3D';y3D'],[z3D';z3D';z3D';z3D']];
    Bbh=msgn*16/315*[x3D,y3D,z3D];
    Bhl=Bh-Bbh*invomega*Zl';
    Bhu=Bh-Bbh*invomega'*Zu';
    Eh=Bbh*invomega*Bbh';
    
    f1t=(f(it1(1))+f(it1(2))+f(it1(3))+f(it1(4)))/4+dxw1*w1t+dyw1*w2t+dzw1*w3t;
    f2t=(f(it2(1))+f(it2(2))+f(it2(3))+f(it2(4)))/4+dxw2*w1t+dyw2*w2t+dzw2*w3t; 
    f3t=(f(it3(1))+f(it3(2))+f(it3(3))+f(it3(4)))/4+dxw3*w1t+dyw3*w2t+dzw3*w3t;
    fh=tvolume/4*[f1t;f1t;f1t;f1t;f2t;f2t;f2t;f2t;f3t;f3t;f3t;f3t];
    fbh=32/105*tvolume*[f1t;f2t;f3t];
    bh=fh-Zu*invomega*fbh;              
    ch=-Bbh*invomega*fbh;
    ah=tvolume/4*e;

    % assembly global data
    A(it123,it123)=A(it123,it123)+Ah;
    Bu(itB,it123)=Bu(itB,it123)+Bhu;
    Bl(itB,it123)=Bl(itB,it123)+Bhl;
    E(itB,itB)=E(itB,itB)+Eh;
    b(it123)=b(it123)+bh;
    c(itB)=c(itB)+ch; a(itB)=a(itB)+ah;
 end