function [A,Bu,Bl,E,b,c,a]=assemblyP1bP1DCN_vec_3(p,t,nu,alpha,f,w)
% Navier-Stokes problem in 3D with P1-bubble/P1 elements for Newton linearization. 
% For symmetric part of gradient.
% Vectorized code.
% 
% Input:
% p ... fem nodes
% t ... fem tetrahedras
% alpha ... constant at mass matrix 
% nu ... viscosity constant
% f ... force at nodes
% w ... given velocity vector
%
% Output:
% A ... alpha*(mass matrix)+nu*(diffusion matrix)+convection matrix(Newton) 
%       for three velocity components u=[u1(1),u2(1),u3(1),u1(2),u2(2),u3(2),...]
% Bu, Bl ... divergence/gradient matrices
% E ... stabilization matrix
% b,c ... components of right-hand side vector
% a ... integral weights over domain
%
% Structure of the system: | A  Bu'| |u| = | b |
%                          | Bl -E | |p|   | c |
% with A = | A11 A12 A13 |
%          | A21 A22 A23 |
%          | A31 A32 A33 |

tt_3=3*t; tt_2=tt_3-1; tt_1=tt_3-2; tt=[tt_1;tt_2;tt_3];
np=size(p,1); np3=3*np;
A=sparse(np3,np3); Bu=sparse(np,np3); Bl=sparse(np,np3); E=sparse(np,np);
b=zeros(np3,1); c=zeros(np,1); a=zeros(np,1);
w1=w(1:3:end-2); w2=w(2:3:end-1); w3=w(3:3:end);
x21=p(t(:,2),1)-p(t(:,1),1); y21=p(t(:,2),2)-p(t(:,1),2); z21=p(t(:,2),3)-p(t(:,1),3);
x31=p(t(:,3),1)-p(t(:,1),1); y31=p(t(:,3),2)-p(t(:,1),2); z31=p(t(:,3),3)-p(t(:,1),3);
x41=p(t(:,4),1)-p(t(:,1),1); y41=p(t(:,4),2)-p(t(:,1),2); z41=p(t(:,4),3)-p(t(:,1),3);
x32=p(t(:,3),1)-p(t(:,2),1); y32=p(t(:,3),2)-p(t(:,2),2); z32=p(t(:,3),3)-p(t(:,2),3);
x42=p(t(:,4),1)-p(t(:,2),1); y42=p(t(:,4),2)-p(t(:,2),2); z42=p(t(:,4),3)-p(t(:,2),3);
x3D=[y42.*z32-y32.*z42 y31.*z41-y41.*z31 y41.*z21-y21.*z41 y21.*z31-y31.*z21];    
y3D=[x32.*z42-x42.*z32 x41.*z31-x31.*z41 x21.*z41-x41.*z21 x31.*z21-x21.*z31];    
z3D=[x42.*y32-x32.*y42 x31.*y41-x41.*y31 x41.*y21-x21.*y41 x21.*y31-x31.*y21];
detX=x21.*x3D(:,2)+x31.*x3D(:,3)+x41.*x3D(:,4); tvolume=detX/6;
msgn=sign(tvolume); tvolume=abs(tvolume);

% for convective matrix (Newton)
w1t=(w1(t(:,1))+w1(t(:,2))+w1(t(:,3))+w1(t(:,4)))/4;
w2t=(w2(t(:,1))+w2(t(:,2))+w2(t(:,3))+w2(t(:,4)))/4;
w3t=(w3(t(:,1))+w3(t(:,2))+w3(t(:,3))+w3(t(:,4)))/4;
dxw1=((w1(t(:,2))-w1(t(:,1))).*x3D(:,2)+(w1(t(:,3))-w1(t(:,1))).*x3D(:,3)+(w1(t(:,4))-w1(t(:,1))).*x3D(:,4))./detX;
dyw1=((w1(t(:,2))-w1(t(:,1))).*y3D(:,2)+(w1(t(:,3))-w1(t(:,1))).*y3D(:,3)+(w1(t(:,4))-w1(t(:,1))).*y3D(:,4))./detX;
dzw1=((w1(t(:,2))-w1(t(:,1))).*z3D(:,2)+(w1(t(:,3))-w1(t(:,1))).*z3D(:,3)+(w1(t(:,4))-w1(t(:,1))).*z3D(:,4))./detX;
dxw2=((w2(t(:,2))-w2(t(:,1))).*x3D(:,2)+(w2(t(:,3))-w2(t(:,1))).*x3D(:,3)+(w2(t(:,4))-w2(t(:,1))).*x3D(:,4))./detX;
dyw2=((w2(t(:,2))-w2(t(:,1))).*y3D(:,2)+(w2(t(:,3))-w2(t(:,1))).*y3D(:,3)+(w2(t(:,4))-w2(t(:,1))).*y3D(:,4))./detX;
dzw2=((w2(t(:,2))-w2(t(:,1))).*z3D(:,2)+(w2(t(:,3))-w2(t(:,1))).*z3D(:,3)+(w2(t(:,4))-w2(t(:,1))).*z3D(:,4))./detX;
dxw3=((w3(t(:,2))-w3(t(:,1))).*x3D(:,2)+(w3(t(:,3))-w3(t(:,1))).*x3D(:,3)+(w3(t(:,4))-w3(t(:,1))).*x3D(:,4))./detX;
dyw3=((w3(t(:,2))-w3(t(:,1))).*y3D(:,2)+(w3(t(:,3))-w3(t(:,1))).*y3D(:,3)+(w3(t(:,4))-w3(t(:,1))).*y3D(:,4))./detX;
dzw3=((w3(t(:,2))-w3(t(:,1))).*z3D(:,2)+(w3(t(:,3))-w3(t(:,1))).*z3D(:,3)+(w3(t(:,4))-w3(t(:,1))).*z3D(:,4))./detX;
w1tx3Dw2ty3Dw3tz3D=w1t.*x3D+w2t.*y3D+w3t.*z3D;

mt1=8/105*tvolume; mt=alpha*mt1; cl=16/315*msgn.*w1tx3Dw2ty3Dw3tz3D;
Zu=mt-cl; 
Zu11=Zu+dxw1.*mt1; 
Zu12=   dyw1.*mt1; 
Zu13=   dzw1.*mt1; 
Zu21=   dxw2.*mt1; Zu22=Zu+dyw2.*mt1; Zu23=   dzw2.*mt1;
Zu31=   dxw3.*mt1; Zu32=   dyw3.*mt1; Zu33=Zu+dzw3.*mt1;
Zl=mt+cl;
Z111=Zl+dxw1.*mt1; Zl22=Zl+dyw2.*mt1; Zl33=Zl+dyw2.*mt1;
Zl12=Zu12; Zl13=Zu13; Zl21=Zu21; Zl23=Zu23; Zl31=Zu31; Zl32=Zu32; 

xyz3D=0.5*(x3D(:,1).^2+y3D(:,1).^2+z3D(:,1).^2 ...
               -x3D(:,2).*(x3D(:,3)+x3D(:,4))-x3D(:,3).*x3D(:,4) ...
               -y3D(:,2).*(y3D(:,3)+y3D(:,4))-y3D(:,3).*y3D(:,4) ...
               -z3D(:,2).*(z3D(:,3)+z3D(:,4))-z3D(:,3).*z3D(:,4));
omegaR=nu*4096/8505./tvolume;
omegaM1=8192/51975*tvolume;
omega11=omegaR.*(xyz3D+0.5*(x3D(:,1).^2-x3D(:,2).*(x3D(:,3)+x3D(:,4))-x3D(:,3).*x3D(:,4)))+(alpha+dxw1).*omegaM1;
omega22=omegaR.*(xyz3D+0.5*(y3D(:,1).^2-y3D(:,2).*(y3D(:,3)+y3D(:,4))-y3D(:,3).*y3D(:,4)))+(alpha+dyw2).*omegaM1;
omega33=omegaR.*(xyz3D+0.5*(z3D(:,1).^2-z3D(:,2).*(z3D(:,3)+z3D(:,4))-z3D(:,3).*z3D(:,4)))+(alpha+dzw3).*omegaM1;
omega12=0.25*omegaR.*(x3D(:,1).*y3D(:,1)+x3D(:,2).*y3D(:,2)+x3D(:,3).*y3D(:,3));
omega21=omega12+dxw2.*omegaM1;
omega12=omega12+dyw1.*omegaM1;
omega13=0.25*omegaR.*(x3D(:,1).*z3D(:,1)+x3D(:,2).*z3D(:,2)+x3D(:,3).*z3D(:,3));
omega31=omega13+dxw3.*omegaM1;
omega13=omega13+dzw1.*omegaM1;
omega23=0.25*omegaR.*(y3D(:,1).*z3D(:,1)+y3D(:,2).*z3D(:,2)+y3D(:,3).*z3D(:,3));
omega32=omega23+dyw3.*omegaM1;
omega23=omega23+dzw2.*omegaM1;

invomega11=omega22.*omega33-omega23.*omega32;  invomega12=-omega12.*omega33+omega32.*omega13; invomega13=omega12.*omega23-omega22.*omega13;
invomega21=-omega21.*omega33+omega31.*omega23; invomega22=omega11.*omega33-omega13.*omega31;  invomega23=-omega11.*omega23+omega21.*omega13;
invomega31= omega21.*omega32-omega31.*omega22; invomega32=-omega11.*omega32+omega31.*omega12; invomega33=omega11.*omega22-omega12.*omega21;
omega_det=omega11.*invomega11+omega12.*invomega21+omega13.*invomega31;
invomega11=invomega11./omega_det; invomega12=invomega12./omega_det; invomega13=invomega13./omega_det;  
invomega21=invomega21./omega_det; invomega22=invomega22./omega_det; invomega23=invomega23./omega_det;
invomega31=invomega31./omega_det; invomega32=invomega32./omega_det; invomega33=invomega33./omega_det;

% Mass matrix
tvolume20=tvolume/20; aux=alpha*tvolume20;
for i=1:4
    for j=1:i
        A=A+sparse(tt(:,i),tt(:,j),[aux;aux;aux],np3,np3);
    end
end
A=A+A';

% Diffusion matrix
n36tv=nu/36./tvolume;
for i=1:4
    n36tvxi=n36tv.*x3D(:,i); n36tvyi=n36tv.*y3D(:,i); n36tvzi=n36tv.*z3D(:,i);
    for j=i:4 % A11, A22, A33 
        n36tvxixj=n36tvxi.*x3D(:,j); n36tvyiyj=n36tvyi.*y3D(:,j); n36tvzizj=n36tvzi.*z3D(:,j);
        aux1=2*n36tvxixj+n36tvyiyj+n36tvzizj; 
        aux2=n36tvxixj+2*n36tvyiyj+n36tvzizj; 
        aux3=n36tvxixj+n36tvyiyj+2*n36tvzizj;
        if i==j
            A=A+sparse(tt(:,i),tt(:,j),[aux1;aux2;aux3],np3,np3);
        else
            A=A+sparse(tt(:,i),tt(:,j),[aux1;aux2;aux3],np3,np3);
            A=A+sparse(tt(:,j),tt(:,i),[aux1;aux2;aux3],np3,np3);
        end
    end
    for j=1:4 % A12, A13, A23, A21, A31, A32
        aux=n36tvyi.*x3D(:,j);
        A=A+sparse(tt_1(:,i),tt_2(:,j),aux,np3,np3)+sparse(tt_2(:,j),tt_1(:,i),aux,np3,np3);
        aux=n36tvzi.*x3D(:,j);
        A=A+sparse(tt_1(:,i),tt_3(:,j),aux,np3,np3)+sparse(tt_3(:,j),tt_1(:,i),aux,np3,np3);
        aux=n36tvzi.*y3D(:,j);
        A=A+sparse(tt_2(:,i),tt_3(:,j),aux,np3,np3)+sparse(tt_3(:,j),tt_2(:,i),aux,np3,np3);
    end
end

% Convection matrix for the Oseen term
aux=(msgn/24).*w1tx3Dw2ty3Dw3tz3D;
for i=1:4 
    for j=1:4
        A=A+sparse(tt(:,i),tt(:,j),[aux(:,j);aux(:,j);aux(:,j)],np3,np3);
    end
end

% Convective matrix for the Newton term
aux11=dxw1.*tvolume20; aux22=dyw2.*tvolume20; aux33=dzw3.*tvolume20; 
aux12=dyw1.*tvolume20; aux21=dxw2.*tvolume20;
aux13=dzw1.*tvolume20; aux31=dxw3.*tvolume20;
aux23=dzw2.*tvolume20; aux32=dyw3.*tvolume20;
for i=1:4
    for j=1:i
        % A11, A22, A33
        A=A+sparse(tt(:,i),tt(:,j),[aux11;aux22;aux33],np3,np3);
        A=A+sparse(tt(:,j),tt(:,i),[aux11;aux22;aux33],np3,np3);
        % A12, A21
        A=A+sparse(tt_1(:,i),tt_2(:,j),aux12,np3,np3);
        A=A+sparse(tt_1(:,j),tt_2(:,i),aux12,np3,np3);
        A=A+sparse(tt_2(:,i),tt_1(:,j),aux21,np3,np3);
        A=A+sparse(tt_2(:,j),tt_1(:,i),aux21,np3,np3);
        % A13, A31
        A=A+sparse(tt_1(:,i),tt_3(:,j),aux13,np3,np3);
        A=A+sparse(tt_1(:,j),tt_3(:,i),aux13,np3,np3);
        A=A+sparse(tt_3(:,i),tt_1(:,j),aux31,np3,np3);
        A=A+sparse(tt_3(:,j),tt_1(:,i),aux31,np3,np3);
        % A23, A32
        A=A+sparse(tt_2(:,i),tt_3(:,j),aux23,np3,np3);
        A=A+sparse(tt_2(:,j),tt_3(:,i),aux23,np3,np3);
        A=A+sparse(tt_3(:,i),tt_2(:,j),aux32,np3,np3);
        A=A+sparse(tt_3(:,j),tt_2(:,i),aux32,np3,np3);
    end
end

% Divergence matrix
for j=1:4
    aux=[(-msgn/24).*x3D(:,j);(-msgn/24).*y3D(:,j);(-msgn/24).*z3D(:,j)];
    for i=1:4
        Bu=Bu+sparse([t(:,i);t(:,i);t(:,i)],tt(:,j),aux,np,np3);
    end
end
Bl=Bu;

% Bubble components elimination in A
mt1=8/105*tvolume; mt=alpha*mt1; 
cl=16/315*msgn.*w1tx3Dw2ty3Dw3tz3D;
Zu=mt-cl; 
Zu11=Zu+dxw1.*mt1; Zu22=Zu+dyw2.*mt1; Zu33=Zu+dzw3.*mt1;
Zu12=dyw1.*mt1; Zu12=[Zu12 Zu12 Zu12 Zu12];
Zu21=dxw2.*mt1; Zu21=[Zu21 Zu21 Zu21 Zu21];
Zu13=dzw1.*mt1; Zu13=[Zu13 Zu13 Zu13 Zu13];
Zu31=dxw3.*mt1; Zu31=[Zu31 Zu31 Zu31 Zu31];
Zu23=dzw2.*mt1; Zu23=[Zu23 Zu23 Zu23 Zu23];
Zu32=dyw3.*mt1; Zu32=[Zu32 Zu32 Zu32 Zu32];
Zl=mt+cl; 
Zl11=Zl+dxw1.*mt1; Zl22=Zl+dyw2.*mt1; Zl33=Zl+dzw3.*mt1;
Zl12=dyw1.*mt1; Zl12=[Zl12 Zl12 Zl12 Zl12];
Zl21=dxw2.*mt1; Zl21=[Zl21 Zl21 Zl21 Zl21];
Zl13=dzw1.*mt1; Zl13=[Zl13 Zl13 Zl13 Zl13];
Zl31=dxw3.*mt1; Zl31=[Zl31 Zl31 Zl31 Zl31];
Zl23=dzw2.*mt1; Zl23=[Zl23 Zl23 Zl23 Zl23];
Zl32=dyw3.*mt1; Zl32=[Zl32 Zl32 Zl32 Zl32];

Zuio11=Zu11.*invomega11+Zu12.*invomega21+Zu13.*invomega31;
Zuio12=Zu11.*invomega12+Zu12.*invomega22+Zu13.*invomega32;
Zuio13=Zu11.*invomega13+Zu12.*invomega23+Zu13.*invomega33;
Zuio21=Zu21.*invomega11+Zu22.*invomega21+Zu23.*invomega31;
Zuio22=Zu21.*invomega12+Zu22.*invomega22+Zu23.*invomega32;
Zuio23=Zu21.*invomega13+Zu22.*invomega23+Zu23.*invomega33;
Zuio31=Zu31.*invomega11+Zu32.*invomega21+Zu33.*invomega31;
Zuio32=Zu31.*invomega12+Zu32.*invomega22+Zu33.*invomega32;
Zuio33=Zu31.*invomega13+Zu32.*invomega23+Zu33.*invomega33;

for i=1:4
    for j=1:4
        aux11=Zuio11(:,i).*Zl11(:,j)+Zuio12(:,i).*Zl12(:,j)+Zuio13(:,i).*Zl13(:,j);
        aux22=Zuio21(:,i).*Zl21(:,j)+Zuio22(:,i).*Zl22(:,j)+Zuio23(:,i).*Zl23(:,j);
        aux33=Zuio31(:,i).*Zl31(:,j)+Zuio32(:,i).*Zl32(:,j)+Zuio33(:,i).*Zl33(:,j);
        aux12=Zuio11(:,i).*Zl21(:,j)+Zuio12(:,i).*Zl22(:,j)+Zuio13(:,i).*Zl23(:,j);
        aux21=Zuio21(:,i).*Zl11(:,j)+Zuio22(:,i).*Zl12(:,j)+Zuio23(:,i).*Zl13(:,j);
        aux13=Zuio11(:,i).*Zl31(:,j)+Zuio12(:,i).*Zl32(:,j)+Zuio13(:,i).*Zl33(:,j);
        aux31=Zuio31(:,i).*Zl11(:,j)+Zuio32(:,i).*Zl12(:,j)+Zuio33(:,i).*Zl13(:,j);
        aux23=Zuio21(:,i).*Zl31(:,j)+Zuio22(:,i).*Zl32(:,j)+Zuio23(:,i).*Zl33(:,j);
        aux32=Zuio31(:,i).*Zl21(:,j)+Zuio32(:,i).*Zl22(:,j)+Zuio33(:,i).*Zl23(:,j);
        A=A-sparse(tt(:,i),tt(:,j),[aux11;aux22;aux33],np3,np3);
        A=A-sparse(tt_1(:,i),tt_2(:,j),aux12,np3,np3);
        A=A-sparse(tt_2(:,i),tt_1(:,j),aux21,np3,np3);
        A=A-sparse(tt_1(:,i),tt_3(:,j),aux13,np3,np3);
        A=A-sparse(tt_3(:,i),tt_1(:,j),aux31,np3,np3);
        A=A-sparse(tt_2(:,i),tt_3(:,j),aux23,np3,np3);
        A=A-sparse(tt_3(:,i),tt_2(:,j),aux32,np3,np3);
    end
end

% Bubble components elimination in Bu, Bl, E
B1b=16/315*msgn.*x3D; B2b=16/315*msgn.*y3D; B3b=16/315*msgn.*z3D;  
B1bio=B1b.*invomega11+B2b.*invomega21+B3b.*invomega31; 
B2bio=B1b.*invomega12+B2b.*invomega22+B3b.*invomega32;
B3bio=B1b.*invomega13+B2b.*invomega23+B3b.*invomega33;
for i=1:4
    for j=1:4
        E=E+sparse(t(:,i),t(:,j),B1bio(:,i).*B1b(:,j)+B2bio(:,i).*B2b(:,j)+B3bio(:,i).*B3b(:,j),np,np);
        aux1=B1b(:,i).*Zuio11(:,j)+B2b(:,i).*Zuio12(:,j)+B3b(:,i).*Zuio13(:,j);
        aux2=B1b(:,i).*Zuio21(:,j)+B2b(:,i).*Zuio22(:,j)+B3b(:,i).*Zuio23(:,j);
        aux3=B1b(:,i).*Zuio31(:,j)+B2b(:,i).*Zuio32(:,j)+B3b(:,i).*Zuio33(:,j);
        Bu=Bu-sparse([t(:,i);t(:,i);t(:,i)],tt(:,j),[aux1;aux2;aux3],np,np3);
        aux1=B1bio(:,i).*Zl11(:,j)+B2bio(:,i).*Zl12(:,j)+B3bio(:,i).*Zl13(:,j);
        aux2=B1bio(:,i).*Zl21(:,j)+B2bio(:,i).*Zl22(:,j)+B3bio(:,i).*Zl23(:,j);
        aux3=B1bio(:,i).*Zl31(:,j)+B2bio(:,i).*Zl32(:,j)+B3bio(:,i).*Zl33(:,j);
        Bl=Bl-sparse([t(:,i);t(:,i);t(:,i)],tt(:,j),[aux1;aux2;aux3],np,np3);
    end
end

% Right hand-side vector b
f1=f(1:3:end-2); f2=f(2:3:end-1); f3=f(3:3:end);
f1=1/4*tvolume.*((f1(t(:,1))+f1(t(:,2))+f1(t(:,3))+f1(t(:,4)))/4+dxw1.*w1t+dyw1.*w2t+dzw1.*w3t); 
f2=1/4*tvolume.*((f2(t(:,1))+f2(t(:,2))+f2(t(:,3))+f2(t(:,4)))/4+dxw2.*w1t+dyw2.*w2t+dzw2.*w3t);
f3=1/4*tvolume.*((f3(t(:,1))+f3(t(:,2))+f3(t(:,3))+f3(t(:,4)))/4+dxw3.*w1t+dyw3.*w2t+dzw3.*w3t);
for i=1:4
    b=b+sparse(tt(:,i),1,[f1;f2;f3],np3,1);
end

% Bubble components elimination in b, c and a
f1b=128/105*f1; f2b=128/105*f2; f3b=128/105*f3;  
for i=1:4
    aux1=Zuio11(:,i).*f1b+Zuio12(:,i).*f2b+Zuio13(:,i).*f3b;
    aux2=Zuio21(:,i).*f1b+Zuio22(:,i).*f2b+Zuio23(:,i).*f3b;
    aux3=Zuio31(:,i).*f1b+Zuio32(:,i).*f2b+Zuio33(:,i).*f3b;
    b=b-sparse(tt(:,i),1,[aux1;aux2;aux3],np3,1);
    aux=B1bio(:,i).*f1b+B2bio(:,i).*f2b+B3bio(:,i).*f3b;
    c=c-sparse(t(:,i),1,aux,np,1);
    a=a+sparse(t(:,i),1,tvolume/4,np,1);
end