function [A,Bu,Bl,E,b,c,a]=assembly3DP1bP1DC_vecO(p,t,nu,alpha,f,w)
% Navier-Stokes problem in 3D with P1-bubble/P1 elements for Oseen linearization. 
% For symmetric part of gradient.
% Vectorized code.
% Optimized version.
%
% Input:
% p ... fem nodes
% t ... fem tetrahedras
% alpha ... constant at mass matrix 
% nu ... viscosity constant
% f ... force at nodes
% w ... given velocity vector
%
% Output:
% A ... alpha*(mass matrix)+nu*(diffusion matrix)+convective matrix(Oseen) 
%       for three velocity components u=[u1(1),u2(1),u3(1),u1(2),u2(2),u3(2),...]
% Bu, Bl ... divergence/gradient matrices
% E ... stabilization matrix
% b,c ... components of right-hand side vector
% a ... integral weights over domain
%
% Structure of the system: | A  Bu'| |u| = | b |
%                          | Bl -E | |p|   | c |
% with A = | A11 A12 A13 |
%          | A21 A22 A23 |
%          | A31 A32 A33 }
% A11=A22=A33 being nonsymmetric, A21=A12', A31=A13', A32=A23'

np=size(p,1);

x21=p(t(:,2),1)-p(t(:,1),1); y21=p(t(:,2),2)-p(t(:,1),2); z21=p(t(:,2),3)-p(t(:,1),3);
x31=p(t(:,3),1)-p(t(:,1),1); y31=p(t(:,3),2)-p(t(:,1),2); z31=p(t(:,3),3)-p(t(:,1),3);
x41=p(t(:,4),1)-p(t(:,1),1); y41=p(t(:,4),2)-p(t(:,1),2); z41=p(t(:,4),3)-p(t(:,1),3);
x32=p(t(:,3),1)-p(t(:,2),1); y32=p(t(:,3),2)-p(t(:,2),2); z32=p(t(:,3),3)-p(t(:,2),3);
x42=p(t(:,4),1)-p(t(:,2),1); y42=p(t(:,4),2)-p(t(:,2),2); z42=p(t(:,4),3)-p(t(:,2),3);
xt=[z32.*y42-y32.*z42,y31.*z41-z31.*y41,z21.*y41-y21.*z41,y21.*z31-z21.*y31];
yt=[x32.*z42-z32.*x42,z31.*x41-x31.*z41,x21.*z41-z21.*x41,z21.*x31-x21.*z31];
zt=[y32.*x42-x32.*y42,x31.*y41-y31.*x41,y21.*x41-x21.*y41,x21.*y31-y21.*x31];

xtx=xt(:,2).*xt(:,3)+xt(:,2).*xt(:,4)+xt(:,3).*xt(:,4);
xty=yt(:,2).*yt(:,3)+yt(:,2).*yt(:,4)+yt(:,3).*yt(:,4);
xtz=zt(:,2).*zt(:,3)+zt(:,2).*zt(:,4)+zt(:,3).*zt(:,4);
xtt1=xtx+(1/2)*xty+(1/2)*xtz;
xtt2=(1/2)*xtx+xty+(1/2)*xtz;
xtt3=(1/2)*xtx+(1/2)*xty+xtz;

tvolume=(x21.*xt(:,2)+x31.*xt(:,3)+x41.*xt(:,4))/6;
msgn=sign(tvolume);
tvolume=abs(tvolume);

nut=nu./tvolume;

omega_11=(8192/51975)*alpha*tvolume+(4096/8505)*nut.*(xt(:,1).^2+(1/2)*yt(:,1).^2+(1/2)*zt(:,1).^2-xtt1);
omega_22=(8192/51975)*alpha*tvolume+(4096/8505)*nut.*((1/2)*xt(:,1).^2+yt(:,1).^2+(1/2)*zt(:,1).^2-xtt2);
omega_33=(8192/51975)*alpha*tvolume+(4096/8505)*nut.*((1/2)*xt(:,1).^2+(1/2)*yt(:,1).^2+zt(:,1).^2-xtt3);
omega_12=(1024/8505)*nut.*(xt(:,1).*yt(:,1)+xt(:,2).*yt(:,2)+xt(:,3).*yt(:,3));
omega_13=(1024/8505)*nut.*(xt(:,1).*zt(:,1)+xt(:,2).*zt(:,2)+xt(:,3).*zt(:,3));
omega_23=(1024/8505)*nut.*(yt(:,1).*zt(:,1)+yt(:,2).*zt(:,2)+yt(:,3).*zt(:,3));

invomega_N=1./(omega_11.*(omega_22.*omega_33-omega_23.*omega_23)...
    +omega_12.*(omega_23.*omega_13-omega_12.*omega_33)...
    +omega_13.*(omega_12.*omega_23-omega_22.*omega_13));

am_11=(omega_22.*omega_33-omega_23.*omega_23).*invomega_N;
am_13=(omega_12.*omega_23-omega_22.*omega_13).*invomega_N;
am_22=(omega_11.*omega_33-omega_13.*omega_13).*invomega_N;
am_31=(omega_12.*omega_23-omega_13.*omega_22).*invomega_N;
am_33=(omega_11.*omega_22-omega_12.*omega_12).*invomega_N;
am_12=(omega_23.*omega_13-omega_12.*omega_33).*invomega_N;
am_21=(omega_13.*omega_23-omega_12.*omega_33).*invomega_N;
am_23=(omega_12.*omega_13-omega_11.*omega_23).*invomega_N;
am_32=(omega_13.*omega_12-omega_11.*omega_23).*invomega_N;


w1=w(1:3:end-2); w2=w(2:3:end-1); w3=w(3:3:end);
w1t=(w1(t(:,1))+w1(t(:,2))+w1(t(:,3))+w1(t(:,4)))/4;
w2t=(w2(t(:,1))+w2(t(:,2))+w2(t(:,3))+w2(t(:,4)))/4;
w3t=(w3(t(:,1))+w3(t(:,2))+w3(t(:,3))+w3(t(:,4)))/4;

cal=(1/24)*msgn.*[w1t.*xt(:,1)+w2t.*yt(:,1)+w3t.*zt(:,1),...
           w1t.*xt(:,2)+w2t.*yt(:,2)+w3t.*zt(:,2),...
           w1t.*xt(:,3)+w2t.*yt(:,3)+w3t.*zt(:,3),...
           w1t.*xt(:,4)+w2t.*yt(:,4)+w3t.*zt(:,4)];

zl = ((8*alpha)/105)*tvolume+(16/315)*msgn.*(w1t.*xt+w2t.*yt+w3t.*zt);
zu = ((16*alpha)/105)*tvolume-zl;

f1=f(1:3:end-2);f2=f(2:3:end-1);f3=f(3:3:end);
f1t=tvolume.*(f1(t(:,1))+f1(t(:,2))+f1(t(:,3))+f1(t(:,4)))/16;
f2t=tvolume.*(f2(t(:,1))+f2(t(:,2))+f2(t(:,3))+f2(t(:,4)))/16;
f3t=tvolume.*(f3(t(:,1))+f3(t(:,2))+f3(t(:,3))+f3(t(:,4)))/16;

f1b = (128/105)*(am_11.*f1t+am_21.*f2t+am_31.*f3t);
f2b = (128/105)*(am_12.*f1t+am_22.*f2t+am_32.*f3t);
f3b = (128/105)*(am_13.*f1t+am_23.*f2t+am_33.*f3t);

A11=sparse(np,np); A12=sparse(np,np); A13=sparse(np,np);
A21=sparse(np,np); A22=sparse(np,np); A23=sparse(np,np);
A31=sparse(np,np); A32=sparse(np,np); A33=sparse(np,np);
Bu1=sparse(np,np); Bu2=sparse(np,np); Bu3=sparse(np,np);
Bl1=sparse(np,np); Bl2=sparse(np,np); Bl3=sparse(np,np);
E=sparse(np,np);
b1=zeros(np,1); b2=zeros(np,1); b3=zeros(np,1); 
c=zeros(np,1); a=zeros(np,1);
for i=1:4
    a=a+sparse(t(:,i),1,tvolume/4,np,1);
    b1=b1+sparse(t(:,i),1,f1t-zu(:,i).*f1b,np,1);
    b2=b2+sparse(t(:,i),1,f2t-zu(:,i).*f2b,np,1);
    b3=b3+sparse(t(:,i),1,f3t-zu(:,i).*f3b,np,1);
    c=c-sparse(t(:,i),1,(16/315).*msgn.*(xt(:,i).*f1b+yt(:,i).*f2b+zt(:,i).*f3b),np,1);

    E1=(16/315)*(am_11.*xt(:,i)+am_21.*yt(:,i)+am_31.*zt(:,i));
    E2=(16/315)*(am_12.*xt(:,i)+am_22.*yt(:,i)+am_32.*zt(:,i));
    E3=(16/315)*(am_13.*xt(:,i)+am_23.*yt(:,i)+am_33.*zt(:,i));
    for j=1:4
        xyz2 = xt(:,i).*xt(:,j)+yt(:,i).*yt(:,j)+zt(:,i).*zt(:,j);
        if(i==j)
            A11=A11+sparse(t(:,i),t(:,j),(alpha/10)*tvolume+(nut/36).*(xyz2+xt(:,i).*xt(:,j))-(zu(:,i).*zl(:,j).*am_11)+cal(:,j),np,np);
            A22=A22+sparse(t(:,i),t(:,j),(alpha/10)*tvolume+(nut/36).*(xyz2+yt(:,i).*yt(:,j))-(zu(:,i).*zl(:,j).*am_22)+cal(:,j),np,np);
            A33=A33+sparse(t(:,i),t(:,j),(alpha/10)*tvolume+(nut/36).*(xyz2+zt(:,i).*zt(:,j))-(zu(:,i).*zl(:,j).*am_33)+cal(:,j),np,np);
        else
            A11=A11+sparse(t(:,i),t(:,j),(alpha/20)*tvolume+(nut/36).*(xyz2+xt(:,i).*xt(:,j))-(zu(:,i).*zl(:,j).*am_11)+cal(:,j),np,np);
            A22=A22+sparse(t(:,i),t(:,j),(alpha/20)*tvolume+(nut/36).*(xyz2+yt(:,i).*yt(:,j))-(zu(:,i).*zl(:,j).*am_22)+cal(:,j),np,np);
            A33=A33+sparse(t(:,i),t(:,j),(alpha/20)*tvolume+(nut/36).*(xyz2+zt(:,i).*zt(:,j))-(zu(:,i).*zl(:,j).*am_33)+cal(:,j),np,np);
        end
        A12=A12+sparse(t(:,i),t(:,j),(nut/36).*(yt(:,i).*xt(:,j))-zu(:,i).*zl(:,j).*am_21,np,np);
        A21=A21+sparse(t(:,i),t(:,j),(nut/36).*(xt(:,i).*yt(:,j))-zu(:,i).*zl(:,j).*am_12,np,np);
        A13=A13+sparse(t(:,i),t(:,j),(nut/36).*(zt(:,i).*xt(:,j))-zu(:,i).*zl(:,j).*am_31,np,np);
        A31=A31+sparse(t(:,i),t(:,j),(nut/36).*(xt(:,i).*zt(:,j))-zu(:,i).*zl(:,j).*am_13,np,np);
        A23=A23+sparse(t(:,i),t(:,j),(nut/36).*(zt(:,i).*yt(:,j))-zu(:,i).*zl(:,j).*am_32,np,np);
        A32=A32+sparse(t(:,i),t(:,j),(nut/36).*(yt(:,i).*zt(:,j))-zu(:,i).*zl(:,j).*am_23,np,np);

        Bu1=Bu1-sparse(t(:,i),t(:,j),msgn.*((1/24)*xt(:,j)+(zu(:,j).*E1)),np,np);
        Bu2=Bu2-sparse(t(:,i),t(:,j),msgn.*((1/24)*yt(:,j)+(zu(:,j).*E2)),np,np);
        Bu3=Bu3-sparse(t(:,i),t(:,j),msgn.*((1/24)*zt(:,j)+(zu(:,j).*E3)),np,np);
        Bl1=Bl1-sparse(t(:,i),t(:,j),msgn.*((1/24)*xt(:,j)+(zl(:,j).*E1)),np,np);
        Bl2=Bl2-sparse(t(:,i),t(:,j),msgn.*((1/24)*yt(:,j)+(zl(:,j).*E2)),np,np);
        Bl3=Bl3-sparse(t(:,i),t(:,j),msgn.*((1/24)*zt(:,j)+(zl(:,j).*E3)),np,np);
        E=E+sparse(t(:,i),t(:,j),(16/315)*(xt(:,j).*E1+yt(:,j).*E2+zt(:,j).*E3),np,np);
    end
end
t3(1:3:3*np-2)=1:np; t3(2:3:3*np-1)=np+(1:np); t3(3:3:3*np)=2*np+(1:np);
A=[A11 A12 A13;
   A21 A22 A23;
   A31 A32 A33]; A=A(t3,t3);
Bu=[Bu1 Bu2 Bu3]; Bu=Bu(:,t3);
Bl=[Bl1 Bl2 Bl3]; Bl=Bl(:,t3);
b=[b1;b2;b3]; b=b(t3,1);
end