function [A,Bu,Bl,E,b,c,a]=assemblyP1bP1C_3(p,t,nu,alpha,f,w)
% Navier-Stokes problem in 3D with P1-bubble/P1 elements for Oseen linearization.
% For Laplace operator.
% Loop over tetrahedras.
%
% Input:
% p ... fem nodes
% t ... fem tetrahedras
% alpha ... constant at mass matrix 
% nu ... viscosity constant
% f ... force at nodes
% w ... given velocity vector
%
% Output:
% A ... alpha*(mass matrix)+nu*(diffusion matrix)+convection matrix(Oseen) 
%       for three velocity components u=[u1(1),u2(1),u3(1),u1(2),u2(2),u3(2),...]
% Bu, Bl ... divergence/gradient matrices
% E ... stabilization matrix
% b,c ... components of right-hand side vector
% a ... integral weights over domain
%
% Structure of the system: | A  Bu'| |u| = | b |
%                          | Bl -E | |p|   | c |
% with A = | A11 0   0   |
%          | 0   A22 0   |
%          | 0   0   A33 |
% A11=A22=A33 being nonsymmetric

np=size(p,1); nt=size(t,1);
A=sparse(3*np,3*np); Bu=sparse(np,3*np); Bl=sparse(np,3*np); E=sparse(np,np);
b=zeros(3*np,1); c=zeros(np,1); a=zeros(np,1);
Mel=1/20*[2 1 1 1; 1 2 1 1; 1 1 2 1; 1 1 1 2];; e=[1;1;1;1];
w1=w(1:3:end-2); w2=w(2:3:end-1); w3=w(3:3:end);

for ih=1:nt
    itB=t(ih,1:4); it1=3*itB-2; it2=it1+1; it3=it1+2;    
    x21=p(itB(2),1)-p(itB(1),1); y21=p(itB(2),2)-p(itB(1),2); z21=p(itB(2),3)-p(itB(1),3);
    x31=p(itB(3),1)-p(itB(1),1); y31=p(itB(3),2)-p(itB(1),2); z31=p(itB(3),3)-p(itB(1),3);
    x41=p(itB(4),1)-p(itB(1),1); y41=p(itB(4),2)-p(itB(1),2); z41=p(itB(4),3)-p(itB(1),3);
    x32=p(itB(3),1)-p(itB(2),1); y32=p(itB(3),2)-p(itB(2),2); z32=p(itB(3),3)-p(itB(2),3);
    x42=p(itB(4),1)-p(itB(2),1); y42=p(itB(4),2)-p(itB(2),2); z42=p(itB(4),3)-p(itB(2),3);
    x3D=[y42*z32-y32*z42; y31*z41-y41*z31; y41*z21-y21*z41; y21*z31-y31*z21];    
    y3D=[x32*z42-x42*z32; x41*z31-x31*z41; x21*z41-x41*z21; x31*z21-x21*z31];    
    z3D=[x42*y32-x32*y42; x31*y41-x41*y31; x41*y21-x21*y41; x21*y31-x31*y21];
    tvolume=(x21*x3D(2)+x31*x3D(3)+x41*x3D(4))/6;
    msgn=sign(tvolume); tvolume=abs(tvolume); altv=alpha*tvolume;
    
    % for convective matrix (Oseen)
    w1t=(w1(t(ih,1))+w1(t(ih,2))+w1(t(ih,3))+w1(t(ih,4)))/4;
    w2t=(w2(t(ih,1))+w2(t(ih,2))+w2(t(ih,3))+w2(t(ih,4)))/4;
    w3t=(w3(t(ih,1))+w3(t(ih,2))+w3(t(ih,3))+w3(t(ih,4)))/4;
    
    mt=8/105*alpha*tvolume*e; cl=16/315*msgn*(w1t*x3D+w2t*y3D+w3t*z3D);
    Zu=mt-cl; Zl=mt+cl;
    xyz3D=x3D(2)*(x3D(3)+x3D(4))+x3D(3)*x3D(4)+y3D(2)*(y3D(3)+y3D(4))+y3D(3)*y3D(4)+z3D(2)*(z3D(3)+z3D(4))+z3D(3)*z3D(4);
    omega=2048/8505/tvolume*nu*(x3D(1)^2+y3D(1)^2+z3D(1)^2-xyz3D)+8192/51975*altv;

    % element matrices and vectors
    R=nu/36/tvolume*(x3D*x3D'+y3D*y3D'+z3D*z3D');
    M=alpha*tvolume*Mel;
    C=msgn/24*(w1t*[x3D';x3D';x3D';x3D']+w2t*[y3D';y3D';y3D';y3D']+w3t*[z3D';z3D';z3D';z3D']);
    Ah=R+C+M-(Zu*Zl')/omega;
    B1hl=msgn*(-1/24*[x3D';x3D';x3D';x3D']-16/315/omega*x3D*Zl');
    B2hl=msgn*(-1/24*[y3D';y3D';y3D';y3D']-16/315/omega*y3D*Zl');
    B3hl=msgn*(-1/24*[z3D';z3D';z3D';z3D']-16/315/omega*z3D*Zl');
    B1hu=msgn*(-1/24*[x3D';x3D';x3D';x3D']-16/315/omega*x3D*Zu');
    B2hu=msgn*(-1/24*[y3D';y3D';y3D';y3D']-16/315/omega*y3D*Zu');
    B3hu=msgn*(-1/24*[z3D';z3D';z3D';z3D']-16/315/omega*z3D*Zu');
    Eh=256/99225/omega*(x3D*x3D'+y3D*y3D'+z3D*z3D');
    f1t=(f(it1(1))+f(it1(2))+f(it1(3))+f(it1(4)))/4;
    f2t=(f(it2(1))+f(it2(2))+f(it2(3))+f(it2(4)))/4;
    f3t=(f(it3(1))+f(it3(2))+f(it3(3))+f(it3(4)))/4;
    f1b=32/105*tvolume*f1t;
    f2b=32/105*tvolume*f2t; 
    f3b=32/105*tvolume*f3t;
    b1h=tvolume/4*f1t*e-f1b/omega*Zu;
    b2h=tvolume/4*f2t*e-f2b/omega*Zu;
    b3h=tvolume/4*f3t*e-f3b/omega*Zu;
    ch=-msgn*16/315/omega*(f1b*x3D+f2b*y3D+f3b*z3D);
    ah=tvolume/4*e;

    % assembly global data
    A(it1,it1)=A(it1,it1)+Ah;  A(it2,it2)=A(it2,it2)+Ah;  A(it3,it3)=A(it3,it3)+Ah;
    Bu(itB,it1)=Bu(itB,it1)+B1hu; Bu(itB,it2)=Bu(itB,it2)+B2hu; Bu(itB,it3)=Bu(itB,it3)+B3hu;
    Bl(itB,it1)=Bl(itB,it1)+B1hl; Bl(itB,it2)=Bl(itB,it2)+B2hl; Bl(itB,it3)=Bl(itB,it3)+B3hl;
    E(itB,itB)=E(itB,itB)+Eh;
    b(it1)=b(it1)+b1h; b(it2)=b(it2)+b2h; b(it3)=b(it3)+b3h;
    c(itB)=c(itB)+ch; a(itB)=a(itB)+ah;
end