function [A,Bu,Bl,E,b,c,a]=assemblyP1bP1C_vec_3(p,t,nu,alpha,f,w)
% Navier-Stokes problem in 3D with P1-bubble/P1 elements for Oseen linearization.
% For Laplace operator.
% Vectorized code.
%
% Input:
% p ... fem nodes
% t ... fem tetrahedras
% alpha ... constant at mass matrix 
% nu ... viscosity constant
% f ... force at nodes
% w ... given velocity vector
%
% Output:
% A ... alpha*(mass matrix)+nu*(diffusion matrix)+convection matrix(Oseen) 
%       for three velocity components u=[u1(1),u2(1),u3(1),u1(2),u2(2),u3(2),...]
% Bu, Bl ... divergence/gradient matrices
% E ... stabilization matrix
% b,c ... components of right-hand side vector
% a ... integral weights over domain
%
% Structure of the system: | A  Bu'| |u| = | b |
%                          | Bl -E | |p|   | c |
% with A = | A11 0   0   |
%          | 0   A22 0   |
%          | 0   0   A33 |
% A11=A22=A33 being nonsymmetric

tt=3*t; tt=[tt-2;tt-1;tt];
np=size(p,1); np3=3*np;
A=sparse(np3,np3); Bu=sparse(np,np3); Bl=sparse(np,np3); E=sparse(np,np);
b=zeros(np3,1); c=zeros(np,1); a=zeros(np,1);
w1=w(1:3:end-2); w2=w(2:3:end-1); w3=w(3:3:end);
x21=p(t(:,2),1)-p(t(:,1),1); y21=p(t(:,2),2)-p(t(:,1),2); z21=p(t(:,2),3)-p(t(:,1),3);
x31=p(t(:,3),1)-p(t(:,1),1); y31=p(t(:,3),2)-p(t(:,1),2); z31=p(t(:,3),3)-p(t(:,1),3);
x41=p(t(:,4),1)-p(t(:,1),1); y41=p(t(:,4),2)-p(t(:,1),2); z41=p(t(:,4),3)-p(t(:,1),3);
x32=p(t(:,3),1)-p(t(:,2),1); y32=p(t(:,3),2)-p(t(:,2),2); z32=p(t(:,3),3)-p(t(:,2),3);
x42=p(t(:,4),1)-p(t(:,2),1); y42=p(t(:,4),2)-p(t(:,2),2); z42=p(t(:,4),3)-p(t(:,2),3);
x3D=[y42.*z32-y32.*z42 y31.*z41-y41.*z31 y41.*z21-y21.*z41 y21.*z31-y31.*z21];    
y3D=[x32.*z42-x42.*z32 x41.*z31-x31.*z41 x21.*z41-x41.*z21 x31.*z21-x21.*z31];    
z3D=[x42.*y32-x32.*y42 x31.*y41-x41.*y31 x41.*y21-x21.*y41 x21.*y31-x31.*y21];
tvolume=(x21.*x3D(:,2)+x31.*x3D(:,3)+x41.*x3D(:,4))/6;
msgn=sign(tvolume); tvolume=abs(tvolume); altv=alpha*tvolume; 

% for convective matrix
w1t=(w1(t(:,1))+w1(t(:,2))+w1(t(:,3))+w1(t(:,4)))/4;
w2t=(w2(t(:,1))+w2(t(:,2))+w2(t(:,3))+w2(t(:,4)))/4;
w3t=(w3(t(:,1))+w3(t(:,2))+w3(t(:,3))+w3(t(:,4)))/4;
w1tx3Dw2ty3Dw3tz3D=w1t.*x3D+w2t.*y3D+w3t.*z3D;

omega=2048/8505/tvolume*nu*(x3D(:,1).^2+y3D(:,1).^2+z3D(:,1).^2 ...
-x3D(:,2).*(x3D(:,3)+x3D(:,4))-x3D(:,3).*x3D(:,4) ...
-y3D(:,2).*(y3D(:,3)+y3D(:,4))-y3D(:,3).*y3D(:,4) ...
-z3D(:,2).*(z3D(:,3)+z3D(:,4))-z3D(:,3).*z3D(:,4)) ...
+8192/51975*altv;

% Mass matrix
aux=altv/20;
for i=1:4
    for j=1:i
        A=A+sparse(tt(:,i),tt(:,j),[aux;aux;aux],np3,np3);
    end
end
A=A+A';

% Diffusion matrix
n36tv=nu/36./tvolume;
for i=1:4
    n36tvxi=n36tv.*x3D(:,i); 
    n36tvyi=n36tv.*y3D(:,i);  
    n36tvzi=n36tv.*z3D(:,i);
    for j=i:4 % A11, A33=A22=A11 
        aux=n36tvxi.*x3D(:,j)+n36tvyi.*y3D(:,j)+n36tvzi.*z3D(:,j);
        if i==j
            A=A+sparse(tt(:,i),tt(:,j),[aux;aux;aux],np3,np3);
        else
            A=A+sparse(tt(:,i),tt(:,j),[aux;aux;aux],np3,np3);
            A=A+sparse(tt(:,j),tt(:,i),[aux;aux;aux],np3,np3);
        end
    end
end

% Convection matrix for the Oseen term
aux=(msgn/24).*w1tx3Dw2ty3Dw3tz3D;
for i=1:4 
    for j=1:4
        A=A+sparse(tt(:,i),tt(:,j),[aux(:,j);aux(:,j);aux(:,j)],np3,np3);
    end
end

% Divergence matrix
for j=1:4
    aux=[(-msgn/24).*x3D(:,j);(-msgn/24).*y3D(:,j);(-msgn/24).*z3D(:,j)];
    for i=1:4
        Bu=Bu+sparse([t(:,i);t(:,i);t(:,i)],tt(:,j),aux,np,np3);
    end
end
Bl=Bu;

% Bubble components elimination in A
mt=8/105*altv; clt=16/315*msgn.*w1tx3Dw2ty3Dw3tz3D;
Zu=mt-clt; Zl=mt+clt; Zuiom=Zu./omega;
for i=1:4
    for j=1:4
        aux=Zuiom(:,i).*Zl(:,j);
        A=A-sparse(tt(:,i),tt(:,j),[aux;aux;aux],np3,np3);
    end
end

% Bubble components elimination in Bu, Bl, E
Bbh1=16/315*msgn.*x3D; Bbh1iom=Bbh1./omega;
Bbh2=16/315*msgn.*y3D; Bbh2iom=Bbh2./omega;
Bbh3=16/315*msgn.*z3D; Bbh3iom=Bbh3./omega;
for i=1:4
    for j=1:4
        aux=Bbh1iom(:,i).*Bbh1(:,j)+Bbh2iom(:,i).*Bbh2(:,j)+Bbh3iom(:,i).*Bbh3(:,j);
        E=E+sparse(t(:,i),t(:,j),aux,np,np);
        aux1=Bbh1iom(:,i).*Zu(:,j); aux2=Bbh2iom(:,i).*Zu(:,j); aux3=Bbh3iom(:,i).*Zu(:,j);
        Bu=Bu-sparse([t(:,i);t(:,i);t(:,i)],tt(:,j),[aux1;aux2;aux3],np,np3);
        aux1=Bbh1iom(:,i).*Zl(:,j); aux2=Bbh2iom(:,i).*Zl(:,j); aux3=Bbh3iom(:,i).*Zl(:,j);
        Bl=Bl-sparse([t(:,i);t(:,i);t(:,i)],tt(:,j),[aux1;aux2;aux3],np,np3);
     end
end

% Right hand-side vector b, c and a
f1=f(1:3:end-2); f2=f(2:3:end-1); f3=f(3:3:end);
f1=tvolume.*(f1(t(:,1))+f1(t(:,2))+f1(t(:,3))+f1(t(:,4)))/16; 
f2=tvolume.*(f2(t(:,1))+f2(t(:,2))+f2(t(:,3))+f2(t(:,4)))/16;
f3=tvolume.*(f3(t(:,1))+f3(t(:,2))+f3(t(:,3))+f3(t(:,4)))/16;
f1b=128/105*f1; f2b=128/105*f2; f3b=128/105*f3;
for i=1:4
    aux1=f1-Zuiom(:,i).*f1b; aux2=f2-Zuiom(:,i).*f2b; aux3=f3-Zuiom(:,i).*f3b;
    b=b+sparse(tt(:,i),1,[aux1;aux2;aux3],np3,1);
    aux=Bbh1iom(:,i).*f1b+Bbh2iom(:,i).*f2b+Bbh3iom(:,i).*f3b;
    c=c-sparse(t(:,i),1,aux,np,1);
    a=a+sparse(t(:,i),1,tvolume/4,np,1);
end