function [A,Bu,Bl,E,b,c,a]=assemblyP1bP1DC_3(p,t,nu,alpha,f,w)
% Navier-Stokes problem in 3D with P1-bubble/P1 elements for Oseen linearization. 
% For symmetric part of gradient.
% Loop over tetrahedras.
%
% Input:
% p ... fem nodes
% t ... fem tetrahedras
% alpha ... constant at mass matrix 
% nu ... viscosity constant
% f ... force at nodes
% w ... given velocity vector
%
% Output:
% A ... alpha*(mass matrix)+nu*(diffusion matrix)+convective matrix(Oseen) 
%       for three velocity components u=[u1(1),u2(1),u3(1),u1(2),u2(2),u3(2),...]
% Bu, Bl ... divergence/gradient matrices
% E ... stabilization matrix
% b,c ... components of right-hand side vector
% a ... integral weights over domain
%
% Structure of the system: | A  Bu'| |u| = | b |
%                          | Bl -E | |p|   | c |
% with A = | A11 A12 A13 |
%          | A21 A22 A23 |
%          | A31 A32 A33 }
% A11=A22=A33 being nonsymmetric, A21=A12', A31=A13', A32=A23' 

np=size(p,1); nt=size(t,1);
A=sparse(3*np,3*np); Bu=sparse(np,3*np); Bl=sparse(np,3*np); E=sparse(np,np);
b=zeros(3*np,1); c=zeros(np,1); a=zeros(np,1);
Mel=1/20*[2 1 1 1; 1 2 1 1; 1 1 2 1; 1 1 1 2]; e=[1;1;1;1];
w1=w(1:3:end-2); w2=w(2:3:end-1); w3=w(3:3:end);

for ih=1:nt
    itB=t(ih,1:4); it1=3*itB-2; it2=it1+1; it3=it1+2; it123=[it1,it2,it3];
    x21=p(itB(2),1)-p(itB(1),1); y21=p(itB(2),2)-p(itB(1),2); z21=p(itB(2),3)-p(itB(1),3);
    x31=p(itB(3),1)-p(itB(1),1); y31=p(itB(3),2)-p(itB(1),2); z31=p(itB(3),3)-p(itB(1),3);
    x41=p(itB(4),1)-p(itB(1),1); y41=p(itB(4),2)-p(itB(1),2); z41=p(itB(4),3)-p(itB(1),3);
    x32=p(itB(3),1)-p(itB(2),1); y32=p(itB(3),2)-p(itB(2),2); z32=p(itB(3),3)-p(itB(2),3);
    x42=p(itB(4),1)-p(itB(2),1); y42=p(itB(4),2)-p(itB(2),2); z42=p(itB(4),3)-p(itB(2),3);
    x3D=[y42*z32-y32*z42; y31*z41-y41*z31; y41*z21-y21*z41; y21*z31-y31*z21];    
    y3D=[x32*z42-x42*z32; x41*z31-x31*z41; x21*z41-x41*z21; x31*z21-x21*z31];    
    z3D=[x42*y32-x32*y42; x31*y41-x41*y31; x41*y21-x21*y41; x21*y31-x31*y21];
    tvolume=(x21*x3D(2)+x31*x3D(3)+x41*x3D(4))/6;
    msgn=sign(tvolume); tvolume=abs(tvolume); altv=alpha*tvolume;

    % for convective matrix (Oseen)
    w1t=(w1(t(ih,1))+w1(t(ih,2))+w1(t(ih,3))+w1(t(ih,4)))/4;
    w2t=(w2(t(ih,1))+w2(t(ih,2))+w2(t(ih,3))+w2(t(ih,4)))/4;
    w3t=(w3(t(ih,1))+w3(t(ih,2))+w3(t(ih,3))+w3(t(ih,4)))/4;
    
    mt=8/105*altv*e; cl=16/315*msgn*(w1t*x3D+w2t*y3D+w3t*z3D);
    Zu=mt-cl; Zl=mt+cl;
    Zu=[Zu,zeros(4,2);zeros(4,1),Zu,zeros(4,1);zeros(4,2),Zu];
    Zl=[Zl,zeros(4,2);zeros(4,1),Zl,zeros(4,1);zeros(4,2),Zl];
    
    xyz3D=0.5*(x3D(1)^2+y3D(1)^2+z3D(1)^2 ...
               -x3D(2)*(x3D(3)+x3D(4))-x3D(3)*x3D(4) ...
               -y3D(2)*(y3D(3)+y3D(4))-y3D(3)*y3D(4) ...
               -z3D(2)*(z3D(3)+z3D(4))-z3D(3)*z3D(4));
    omegaR=nu*4096/8505/tvolume;
    omegaM=8192/51975*altv;
    omega11=omegaR*(xyz3D+0.5*(x3D(1)^2-x3D(2)*(x3D(3)+x3D(4))-x3D(3)*x3D(4)))+omegaM;
    omega22=omegaR*(xyz3D+0.5*(y3D(1)^2-y3D(2)*(y3D(3)+y3D(4))-y3D(3)*y3D(4)))+omegaM;
    omega33=omegaR*(xyz3D+0.5*(z3D(1)^2-z3D(2)*(z3D(3)+z3D(4))-z3D(3)*z3D(4)))+omegaM;
    omega12=0.25*omegaR*(x3D(1)*y3D(1)+x3D(2)*y3D(2)+x3D(3)*y3D(3));
    omega13=0.25*omegaR*(x3D(1)*z3D(1)+x3D(2)*z3D(2)+x3D(3)*z3D(3));
    omega23=0.25*omegaR*(y3D(1)*z3D(1)+y3D(2)*z3D(2)+y3D(3)*z3D(3));
    invomega=[omega22*omega33-omega23^2, -omega12*omega33+omega23*omega13, omega12*omega23-omega22*omega13
              -omega12*omega33+omega13*omega23, omega11*omega33-omega13^2, -omega11*omega23+omega12*omega13
              omega12*omega23-omega13*omega22, -omega11*omega23+omega13*omega12, omega11*omega22-omega12^2];
    omega_det=omega11*invomega(1,1)+omega12*invomega(2,1)+omega13*invomega(3,1);
    invomega=invomega/omega_det;

    % element matrices and vectors
    M=altv*Mel;
    C=msgn/24*(w1t*[x3D';x3D';x3D';x3D']+w2t*[y3D';y3D';y3D';y3D']+w3t*[z3D';z3D';z3D';z3D']);
    xyz3D=0.5*(x3D*x3D'+y3D*y3D'+z3D*z3D');
    Ah=[C+M+nu/18/tvolume*(xyz3D+0.5*x3D*x3D') nu/36/tvolume*y3D*x3D'                 nu/36/tvolume*z3D*x3D'
        nu/36/tvolume*x3D*y3D'                 C+M+nu/18/tvolume*(xyz3D+0.5*y3D*y3D') nu/36/tvolume*z3D*y3D'
        nu/36/tvolume*x3D*z3D'                 nu/36/tvolume*y3D*z3D'                 C+M+nu/18/tvolume*(xyz3D+0.5*z3D*z3D')];    
    Ah=Ah-Zu*invomega*Zl';
    Bh=-msgn/24*[[x3D';x3D';x3D';x3D'],[y3D';y3D';y3D';y3D'],[z3D';z3D';z3D';z3D']];
    Bbh=msgn*16/315*[x3D,y3D,z3D];
    Bhl=Bh-Bbh*invomega*Zl';
    Bhu=Bh-Bbh*invomega*Zu';
    Eh=Bbh*invomega*Bbh';
    
    % element vectors
    f1t=(f(it1(1))+f(it1(2))+f(it1(3))+f(it1(4)))/4;
    f2t=(f(it2(1))+f(it2(2))+f(it2(3))+f(it2(4)))/4; 
    f3t=(f(it3(1))+f(it3(2))+f(it3(3))+f(it3(4)))/4;
    fh=tvolume/4*[f1t;f1t;f1t;f1t;f2t;f2t;f2t;f2t;f3t;f3t;f3t;f3t];
    fbh=32/105*tvolume*[f1t;f2t;f3t];
    bh=fh-Zu*invomega*fbh;              
    ch=-Bbh*invomega*fbh;
    ah=tvolume/4*e;

    % assembly global data
    A(it123,it123)=A(it123,it123)+Ah;
    Bu(itB,it123)=Bu(itB,it123)+Bhu;
    Bl(itB,it123)=Bl(itB,it123)+Bhl;
    E(itB,itB)=E(itB,itB)+Eh;
    b(it123)=b(it123)+bh;
    c(itB)=c(itB)+ch; a(itB)=a(itB)+ah;
end