function [A,B,E,b,c,a]=assembly3DP1bP1D_vecO(p,t,nu,alpha,f)
% Stokes problem in 3D with P1-bubble/P1 elements.
% For symmetric part of the gradient.
% Vectorized code.
% Optimized version.
%
% Input:
% p ... fem nodes
% t ... fem tetrahedras
% alpha ... constant at mass matrix
% nu ... viscosity constant
% f ... force at nodes
%
% Output:
% A ... alpha*(mass matrix)+nu*(diffusion matrix)
%       for three velocity components u=[u1(1),u2(1),u3(1),u1(2),u2(2),u3(2),...]
% B ... divergence matrix
% E ... stabilization matrix
% b,c ... components of right-hand side vector
% a ... integral weights over domain
%
% Structure of the system: | A  B'| |u| = | b |
%                          | B -E | |p|   | c |
% with A = | A11 A12 A13 |
%          | A21 A22 A23 |
%          | A31 A32 A33 }
% being symmetric

np=size(p,1);

x21=p(t(:,2),1)-p(t(:,1),1); y21=p(t(:,2),2)-p(t(:,1),2); z21=p(t(:,2),3)-p(t(:,1),3);
x31=p(t(:,3),1)-p(t(:,1),1); y31=p(t(:,3),2)-p(t(:,1),2); z31=p(t(:,3),3)-p(t(:,1),3);
x41=p(t(:,4),1)-p(t(:,1),1); y41=p(t(:,4),2)-p(t(:,1),2); z41=p(t(:,4),3)-p(t(:,1),3);
x32=p(t(:,3),1)-p(t(:,2),1); y32=p(t(:,3),2)-p(t(:,2),2); z32=p(t(:,3),3)-p(t(:,2),3);
x42=p(t(:,4),1)-p(t(:,2),1); y42=p(t(:,4),2)-p(t(:,2),2); z42=p(t(:,4),3)-p(t(:,2),3);
xt=[z32.*y42-y32.*z42,y31.*z41-z31.*y41,z21.*y41-y21.*z41,y21.*z31-z21.*y31];
yt=[x32.*z42-z32.*x42,z31.*x41-x31.*z41,x21.*z41-z21.*x41,z21.*x31-x21.*z31];
zt=[y32.*x42-x32.*y42,x31.*y41-y31.*x41,y21.*x41-x21.*y41,x21.*y31-y21.*x31];

xtx=xt(:,2).*xt(:,3)+xt(:,2).*xt(:,4)+xt(:,3).*xt(:,4);
xty=yt(:,2).*yt(:,3)+yt(:,2).*yt(:,4)+yt(:,3).*yt(:,4);
xtz=zt(:,2).*zt(:,3)+zt(:,2).*zt(:,4)+zt(:,3).*zt(:,4);
xtt1=xtx+(1/2)*xty+(1/2)*xtz;
xtt2=(1/2)*xtx+xty+(1/2)*xtz;
xtt3=(1/2)*xtx+(1/2)*xty+xtz;

tvolume=(x21.*xt(:,2)+x31.*xt(:,3)+x41.*xt(:,4))/6;
msgn=sign(tvolume);
tvolume=abs(tvolume);

nut=nu./tvolume;
mt=(8/105)*alpha*tvolume;

omega_11=(8192/51975)*alpha*tvolume+(4096/8505)*nut.*(xt(:,1).^2+(1/2)*yt(:,1).^2+(1/2)*zt(:,1).^2-xtt1);
omega_22=(8192/51975)*alpha*tvolume+(4096/8505)*nut.*((1/2)*xt(:,1).^2+yt(:,1).^2+(1/2)*zt(:,1).^2-xtt2);
omega_33=(8192/51975)*alpha*tvolume+(4096/8505)*nut.*((1/2)*xt(:,1).^2+(1/2)*yt(:,1).^2+zt(:,1).^2-xtt3);

omega_12=(1024/8505)*nut.*(xt(:,1).*yt(:,1)+xt(:,2).*yt(:,2)+xt(:,3).*yt(:,3));
omega_13=(1024/8505)*nut.*(xt(:,1).*zt(:,1)+xt(:,2).*zt(:,2)+xt(:,3).*zt(:,3));
omega_23=(1024/8505)*nut.*(yt(:,1).*zt(:,1)+yt(:,2).*zt(:,2)+yt(:,3).*zt(:,3));

invomega_N=1./(omega_11.*(omega_22.*omega_33-omega_23.*omega_23)...
    +omega_12.*(omega_23.*omega_13-omega_12.*omega_33)...
    +omega_13.*(omega_12.*omega_23-omega_22.*omega_13));

am_11=(omega_22.*omega_33-omega_23.*omega_23).*invomega_N;
am_13=(omega_12.*omega_23-omega_22.*omega_13).*invomega_N;
am_22=(omega_11.*omega_33-omega_13.*omega_13).*invomega_N;
am_31=(omega_12.*omega_23-omega_13.*omega_22).*invomega_N;
am_33=(omega_11.*omega_22-omega_12.*omega_12).*invomega_N;
am_12=(omega_23.*omega_13-omega_12.*omega_33).*invomega_N;
am_21=(omega_13.*omega_23-omega_12.*omega_33).*invomega_N;
am_23=(omega_12.*omega_13-omega_11.*omega_23).*invomega_N;
am_32=(omega_13.*omega_12-omega_11.*omega_23).*invomega_N;


f1=f(1:3:end-2);f2=f(2:3:end-1);f3=f(3:3:end);
f1t=tvolume.*(f1(t(:,1))+f1(t(:,2))+f1(t(:,3))+f1(t(:,4)))/16;
f2t=tvolume.*(f2(t(:,1))+f2(t(:,2))+f2(t(:,3))+f2(t(:,4)))/16;
f3t=tvolume.*(f3(t(:,1))+f3(t(:,2))+f3(t(:,3))+f3(t(:,4)))/16;

f1=(128/105)*(am_11.*f1t+am_21.*f2t+am_31.*f3t);
f2=(128/105)*(am_12.*f1t+am_22.*f2t+am_32.*f3t);
f3=(128/105)*(am_13.*f1t+am_23.*f2t+am_33.*f3t);

f1t=f1t-mt.*f1;
f2t=f2t-mt.*f2;
f3t=f3t-mt.*f3;

A11=sparse(np,np); A12=sparse(np,np); A13=sparse(np,np);
A21=sparse(np,np); A22=sparse(np,np); A23=sparse(np,np);
A31=sparse(np,np); A32=sparse(np,np); A33=sparse(np,np);
B1=sparse(np,np); B2=sparse(np,np); B3=sparse(np,np);
E=sparse(np,np);
b1=zeros(np,1); b2=zeros(np,1); b3=zeros(np,1); 
c=zeros(np,1);a=zeros(np,1);
for i=1:4
    a=a+sparse(t(:,i),1,tvolume/4,np,1);
    b1=b1+sparse(t(:,i),1,f1t,np,1);
    b2=b2+sparse(t(:,i),1,f2t,np,1);
    b3=b3+sparse(t(:,i),1,f3t,np,1);
    c=c-sparse(t(:,i),1,((16/315)*msgn.*(xt(:,i).*f1+yt(:,i).*f2+zt(:,i).*f3)),np,1);

    E1=(16/315)*(am_11.*xt(:,i)+am_21.*yt(:,i)+am_31.*zt(:,i));
    E2=(16/315)*(am_12.*xt(:,i)+am_22.*yt(:,i)+am_32.*zt(:,i));
    E3=(16/315)*(am_13.*xt(:,i)+am_23.*yt(:,i)+am_33.*zt(:,i));
    for j=1:4
        xyz2 = xt(:,i).*xt(:,j)+yt(:,i).*yt(:,j)+zt(:,i).*zt(:,j);
        if(i==j)
            A11=A11+sparse(t(:,i),t(:,j),(alpha/10)*tvolume+(nut/36).*(xyz2+xt(:,i).*xt(:,j))-(mt.^2).*am_11,np,np);
            A22=A22+sparse(t(:,i),t(:,j),(alpha/10)*tvolume+(nut/36).*(xyz2+yt(:,i).*yt(:,j))-(mt.^2).*am_22,np,np);
            A33=A33+sparse(t(:,i),t(:,j),(alpha/10)*tvolume+(nut/36).*(xyz2+zt(:,i).*zt(:,j))-(mt.^2).*am_33,np,np);
        else
            A11=A11+sparse(t(:,i),t(:,j),(alpha/20)*tvolume+(nut/36).*(xyz2+xt(:,i).*xt(:,j))-(mt.^2).*am_11,np,np);
            A22=A22+sparse(t(:,i),t(:,j),(alpha/20)*tvolume+(nut/36).*(xyz2+yt(:,i).*yt(:,j))-(mt.^2).*am_22,np,np);
            A33=A33+sparse(t(:,i),t(:,j),(alpha/20)*tvolume+(nut/36).*(xyz2+zt(:,i).*zt(:,j))-(mt.^2).*am_33,np,np);
        end
        A12=A12+sparse(t(:,i),t(:,j),(nut/36).*(yt(:,i).*xt(:,j))-(mt.^2).*am_21,np,np);
        A13=A13+sparse(t(:,i),t(:,j),(nut/36).*(zt(:,i).*xt(:,j))-(mt.^2).*am_31,np,np);
        A21=A21+sparse(t(:,i),t(:,j),(nut/36).*(xt(:,i).*yt(:,j))-(mt.^2).*am_12,np,np);
        A23=A23+sparse(t(:,i),t(:,j),(nut/36).*(zt(:,i).*yt(:,j))-(mt.^2).*am_32,np,np);
        A31=A31+sparse(t(:,i),t(:,j),(nut/36).*(xt(:,i).*zt(:,j))-(mt.^2).*am_13,np,np);
        A32=A32+sparse(t(:,i),t(:,j),(nut/36).*(yt(:,i).*zt(:,j))-(mt.^2).*am_23,np,np);

        B1=B1-sparse(t(:,i),t(:,j),msgn.*((1/24)*xt(:,j)+mt.*E1),np,np);
        B2=B2-sparse(t(:,i),t(:,j),msgn.*((1/24)*yt(:,j)+mt.*E2),np,np);
        B3=B3-sparse(t(:,i),t(:,j),msgn.*((1/24)*zt(:,j)+mt.*E3),np,np);
        E=E+sparse(t(:,i),t(:,j),(16/315)*(xt(:,j).*E1+yt(:,j).*E2+zt(:,j).*E3),np,np);
    end
end
t3(1:3:3*np-2)=1:np; t3(2:3:3*np-1)=np+(1:np); t3(3:3:3*np)=2*np+(1:np);
A=[A11 A12 A13;
   A21 A22 A23;
   A31 A32 A33]; A=A(t3,t3);
B=[B1 B2 B3]; B=B(:,t3);
b=[b1;b2;b3]; b=b(t3,1);
end