function [A,B,E,b,c,a] = assembly3DP1bP1_vecO(p,t,nu,alpha,f)
% Stokes problem in 3D with P1-bubble/P1 elements.
% For Laplace operator.
% Vectorized code.
% Optimized version.
%
% Input:
% p ... fem nodes
% t ... fem tetrahedras
% alpha ... constant at mass matrix
% nu ... viscosity constant
% f ... force at nodes
%
% Output:
% A ... alpha*(mass matrix)+nu*(diffusion matrix)
%       for three velocity components u=[u1(1),u2(1),u3(1),u1(2),u2(2),u3(2),...]
% B ... divergence matrix
% E ... stabilization matrix
% b,c ... components of right-hand side vector
% a ... integral weights over domain
%
% Structure of the system: | A  B'| |u| = | b |
%                          | B -E | |p|   | c |
% with A = | A11 0   0   |
%          | 0   A22 0   |
%          | 0   0   A33 |
% A11=A22=A33 being symmetric

np=size(p,1);

x21=p(t(:,2),1)-p(t(:,1),1); y21=p(t(:,2),2)-p(t(:,1),2); z21=p(t(:,2),3)-p(t(:,1),3);
x31=p(t(:,3),1)-p(t(:,1),1); y31=p(t(:,3),2)-p(t(:,1),2); z31=p(t(:,3),3)-p(t(:,1),3);
x41=p(t(:,4),1)-p(t(:,1),1); y41=p(t(:,4),2)-p(t(:,1),2); z41=p(t(:,4),3)-p(t(:,1),3);
x32=p(t(:,3),1)-p(t(:,2),1); y32=p(t(:,3),2)-p(t(:,2),2); z32=p(t(:,3),3)-p(t(:,2),3);
x42=p(t(:,4),1)-p(t(:,2),1); y42=p(t(:,4),2)-p(t(:,2),2); z42=p(t(:,4),3)-p(t(:,2),3);
xt=[z32.*y42-y32.*z42,y31.*z41-z31.*y41,z21.*y41-y21.*z41,y21.*z31-z21.*y31];
yt=[x32.*z42-z32.*x42,z31.*x41-x31.*z41,x21.*z41-z21.*x41,z21.*x31-x21.*z31];
zt=[y32.*x42-x32.*y42,x31.*y41-y31.*x41,y21.*x41-x21.*y41,x21.*y31-y21.*x31];

xtt=xt(:,2).*xt(:,3)+xt(:,2).*xt(:,4)+xt(:,3).*xt(:,4)...
    +yt(:,2).*yt(:,3)+yt(:,2).*yt(:,4)+yt(:,3).*yt(:,4)...
    +zt(:,2).*zt(:,3)+zt(:,2).*zt(:,4)+zt(:,3).*zt(:,4);
tvolume=(x21.*xt(:,2)+x31.*xt(:,3)+x41.*xt(:,4))/6;
msgn=sign(tvolume);
tvolume=abs(tvolume);

mt=(8/105)*alpha*tvolume;
nut=nu./tvolume;
%invomega=1./((8192/51975)*alpha*tvolume+(2048/8505)*nu*(xt(:,1).^2+yt(:,1).^2+zt(:,1).^2-xtt)./tvolume);
invomega=88409475./(2048*((6804*alpha)*tvolume+(10395*nut).*(xt(:,1).^2+yt(:,1).^2+zt(:,1).^2-xtt)));

f1=f(1:3:end-2);f2=f(2:3:end-1);f3=f(3:3:end);
f1t=tvolume.*(f1(t(:,1))+f1(t(:,2))+f1(t(:,3))+f1(t(:,4)))/16;
f2t=tvolume.*(f2(t(:,1))+f2(t(:,2))+f2(t(:,3))+f2(t(:,4)))/16;
f3t=tvolume.*(f3(t(:,1))+f3(t(:,2))+f3(t(:,3))+f3(t(:,4)))/16;

f1=(128/105)*f1t.*invomega;
f2=(128/105)*f2t.*invomega;
f3=(128/105)*f3t.*invomega;

f1t=f1t-mt.*f1;
f2t=f2t-mt.*f2;
f3t=f3t-mt.*f3;

Ah=sparse(np,np); E=sparse(np,np);
B1=sparse(np,np); B2=sparse(np,np); B3=sparse(np,np);
b1=zeros(np,1); b2=zeros(np,1); b3=zeros(np,1);
c=zeros(np,1); a=zeros(np,1);
for i=1:4
    a=a+sparse(t(:,i),1,tvolume/4,np,1);
    b1=b1+sparse(t(:,i),1,f1t,np,1);
    b2=b2+sparse(t(:,i),1,f2t,np,1);
    b3=b3+sparse(t(:,i),1,f3t,np,1);
    c=c-sparse(t(:,i),1,(16/315)*msgn.*(xt(:,i).*f1+yt(:,i).*f2+zt(:,i).*f3),np,1);

    E1=(16/315)*xt(:,i).*invomega;
    E2=(16/315)*yt(:,i).*invomega;
    E3=(16/315)*zt(:,i).*invomega;
    for j=1:4
        if(i==j)
            Ah=Ah+sparse(t(:,i),t(:,j),(alpha/10)*tvolume+(nut/36).*(xt(:,i).*xt(:,j)+yt(:,i).*yt(:,j)...
                +zt(:,i).*zt(:,j))-(mt.^2).*invomega,np,np);
        else
            Ah=Ah+sparse(t(:,i),t(:,j),(alpha/20)*tvolume+(nut/36).*(xt(:,i).*xt(:,j)+yt(:,i).*yt(:,j)...
                +zt(:,i).*zt(:,j))-(mt.^2).*invomega,np,np);
        end
        B1=B1-sparse(t(:,i),t(:,j),msgn.*((1/24)*xt(:,j)+mt.*E1),np,np);
        B2=B2-sparse(t(:,i),t(:,j),msgn.*((1/24)*yt(:,j)+mt.*E2),np,np);
        B3=B3-sparse(t(:,i),t(:,j),msgn.*((1/24)*zt(:,j)+mt.*E3),np,np);
        E=E+sparse(t(:,i),t(:,j),(16/315)*(E1.*xt(:,j)+E2.*yt(:,j)+E3.*zt(:,j)),np,np);
    end
end
t3(1:3:3*np-2)=1:np; t3(2:3:3*np-1)=np+(1:np); t3(3:3:3*np)=2*np+(1:np);
A=[Ah sparse(np,np) sparse(np,np);
   sparse(np,np) Ah sparse(np,np);
   sparse(np,np) sparse(np,np) Ah]; A=A(t3,t3);
B=[B1 B2 B3]; B=B(:,t3);
b=[b1;b2;b3]; b=b(t3,1);
end