function [A,B,E,b,c,a]=assemblyP1bP1_vec_3(p,t,nu,alpha,f)
% Stokes problem in 3D with P1-bubble/P1 elements.
% For Laplace operator.
% Vectorized code.
%
% Input:
% p ... fem nodes
% t ... fem tetrahedras
% alpha ... constant at mass matrix 
% nu ... viscosity constant
% f ... force at nodes
%
% Output:
% A ... alpha*(mass matrix)+nu*(diffusion matrix) 
%       for three velocity components u=[u1(1),u2(1),u3(1),u1(2),u2(2),u3(2),...]
% B ... divergence matrix
% E ... stabilization matrix
% b,c ... components of right-hand side vector
% a ... integral weights over domain
%
% Structure of the system: | A  B'| |u| = | b |
%                          | B -E | |p|   | c |
% with A = | A11 0   0   |
%          | 0   A22 0   |
%          | 0   0   A33 |
% A11=A22=A33 being symmetric

tt=3*t; tt=[tt-2;tt-1;tt];
np=size(p,1); np3=3*np;
A=sparse(np3,np3); B=sparse(np,np3); E=sparse(np,np);
b=zeros(np3,1); c=zeros(np,1); a=zeros(np,1);
x21=p(t(:,2),1)-p(t(:,1),1); y21=p(t(:,2),2)-p(t(:,1),2); z21=p(t(:,2),3)-p(t(:,1),3);
x31=p(t(:,3),1)-p(t(:,1),1); y31=p(t(:,3),2)-p(t(:,1),2); z31=p(t(:,3),3)-p(t(:,1),3);
x41=p(t(:,4),1)-p(t(:,1),1); y41=p(t(:,4),2)-p(t(:,1),2); z41=p(t(:,4),3)-p(t(:,1),3);
x32=p(t(:,3),1)-p(t(:,2),1); y32=p(t(:,3),2)-p(t(:,2),2); z32=p(t(:,3),3)-p(t(:,2),3);
x42=p(t(:,4),1)-p(t(:,2),1); y42=p(t(:,4),2)-p(t(:,2),2); z42=p(t(:,4),3)-p(t(:,2),3);
x3D=[y42.*z32-y32.*z42 y31.*z41-y41.*z31 y41.*z21-y21.*z41 y21.*z31-y31.*z21];    
y3D=[x32.*z42-x42.*z32 x41.*z31-x31.*z41 x21.*z41-x41.*z21 x31.*z21-x21.*z31];    
z3D=[x42.*y32-x32.*y42 x31.*y41-x41.*y31 x41.*y21-x21.*y41 x21.*y31-x31.*y21];
tvolume=(x21.*x3D(:,2)+x31.*x3D(:,3)+x41.*x3D(:,4))/6;
msgn=sign(tvolume); tvolume=abs(tvolume); altv=alpha*tvolume; 
omega=2048/8505/tvolume*nu*(x3D(:,1).^2+y3D(:,1).^2+z3D(:,1).^2 ...
-x3D(:,2).*(x3D(:,3)+x3D(:,4))-x3D(:,3).*x3D(:,4) ...
-y3D(:,2).*(y3D(:,3)+y3D(:,4))-y3D(:,3).*y3D(:,4) ...
-z3D(:,2).*(z3D(:,3)+z3D(:,4))-z3D(:,3).*z3D(:,4)) ...
+8192/51975*altv;

% Mass matrix
aux=altv/20;
for i=1:4
    for j=1:i
        A=A+sparse(tt(:,i),tt(:,j),[aux;aux;aux],np3,np3);
    end
end
A=A+A';

% Diffusion matrix
n36tv=nu/36./tvolume;
for i=1:4
    n36tvxi=n36tv.*x3D(:,i); 
    n36tvyi=n36tv.*y3D(:,i);  
    n36tvzi=n36tv.*z3D(:,i);
    for j=i:4 % A11, A33=A22=A11 
        aux=n36tvxi.*x3D(:,j)+n36tvyi.*y3D(:,j)+n36tvzi.*z3D(:,j);
        if i==j
            A=A+sparse(tt(:,i),tt(:,j),[aux;aux;aux],np3,np3);
        else
            A=A+sparse(tt(:,i),tt(:,j),[aux;aux;aux],np3,np3);
            A=A+sparse(tt(:,j),tt(:,i),[aux;aux;aux],np3,np3);
        end
    end
end

% Divergence matrix
for j=1:4
    aux=[(-msgn/24).*x3D(:,j);(-msgn/24).*y3D(:,j);(-msgn/24).*z3D(:,j)];
    for i=1:4
        B=B+sparse([t(:,i);t(:,i);t(:,i)],tt(:,j),aux,np,np3);
    end
end

% Bubble components elimination in A
Z=8/105*altv; aux=Z.*Z./omega;
for i=1:4
    for j=1:4
        A=A-sparse(tt(:,i),tt(:,j),[aux;aux;aux],np3,np3);
    end
end

% Bubble components elimination in B, E 
aux=16/315*msgn./omega;
for i=1:4
    aux1=aux.*x3D(:,i); aux1z=aux1.*Z;
    aux2=aux.*y3D(:,i); aux2z=aux2.*Z;
    aux3=aux.*z3D(:,i); aux3z=aux3.*Z;
    for j=1:4
        E=E+sparse(t(:,i),t(:,j),16/315*msgn.*(aux1.*x3D(:,j)+aux2.*y3D(:,j)+aux3.*z3D(:,j)),np,np);
        B=B-sparse([t(:,i);t(:,i);t(:,i)],tt(:,j),[aux1z;aux2z;aux3z],np,np3);
    end
end

% Right hand-side vector b
f1=f(1:3:np3); f2=f(2:3:np3); f3=f(3:3:np3);
f1=tvolume.*(f1(t(:,1))+f1(t(:,2))+f1(t(:,3))+f1(t(:,4)))/16;
f2=tvolume.*(f2(t(:,1))+f2(t(:,2))+f2(t(:,3))+f2(t(:,4)))/16;
f3=tvolume.*(f3(t(:,1))+f3(t(:,2))+f3(t(:,3))+f3(t(:,4)))/16; 
for i=1:4
    b=b+sparse(tt(:,i),1,[f1;f2;f3],np3,1);
end

% Bubble components elimination in b, c and a
aux=128/105./omega; f1b=aux.*f1; f2b=aux.*f2; f3b=aux.*f3; 
for i=1:4
    b=b-sparse(tt(:,i),1,[Z.*f1b;Z.*f2b;Z.*f3b],np3,1);
    c=c-sparse(t(:,i),1,(16/315*msgn).*(f1b.*x3D(:,i)+f2b.*y3D(:,i)+f3b.*z3D(:,i)),np,1);
    a=a+sparse(t(:,i),1,tvolume/4,np,1);
end